@@ -1,15 +1,16 @@
 /*
-Copyright (C) 2014- The University of Notre Dame
+Copyright (C) 2024 The University of Notre Dame
 This software is distributed under the GNU General Public License.
 See the file COPYING for details.
 */
 
 #include "work_queue_catalog.h"
+#include "work_queue_protocol.h"
+
 #include "cctools.h"
-#include "batch_job.h"
+#include "batch_queue.h"
 #include "hash_table.h"
 #include "copy_stream.h"
-#include "copy_tree.h"
 #include "debug.h"
 #include "domain_name_cache.h"
 #include "envtools.h"
@@ -17,19 +18,22 @@ See the file COPYING for details.
 #include "xxmalloc.h"
 #include "itable.h"
 #include "create_dir.h"
-#include "delete_dir.h"
+#include "unlink_recursive.h"
 #include "macros.h"
 #include "catalog_query.h"
 #include "link.h"
+#include "link_auth.h"
 #include "list.h"
 #include "get_line.h"
 #include "getopt.h"
 #include "path.h"
 #include "buffer.h"
 #include "rmsummary.h"
+#include "username.h"
 
 #include "jx.h"
 #include "jx_parse.h"
+#include "jx_eval.h"
 #include "jx_print.h"
 #include "jx_table.h"
 
@@ -46,8 +50,7 @@ See the file COPYING for details.
 #include <unistd.h>
 #include <signal.h>
 
-#define CCTOOLS_RUNOS_PATH "/afs/crc.nd.edu/group/ccl/software/runos/runos.py"
-#define CCTOOLS_VC3_BUILDER_PATH "/afs/crc.nd.edu/group/ccl/software/vc3-builder-src/vc3-builder"
+#define CCTOOLS_RUNOS_PATH "/afs/crc.nd.edu/group/ccl/software/runos/runos"
 
 typedef enum {
 	FORMAT_TABLE,
@@ -91,15 +94,19 @@ int using_catalog = 0;
 static char *extra_worker_args=0;
 static char *resource_args=0;
 static int abort_flag = 0;
+static pid_t initial_ppid = 0;
 static const char *scratch_dir = 0;
-static const char *password_file = 0;
 static char *config_file = 0;
 static char *amazon_config = NULL;
 static char *condor_requirements = NULL;
 static char *batch_submit_options = NULL;
+static const char *password_file = 0;
+char *password;
 
 static char *wrapper_command = 0;
-static char *wrapper_input = 0;
+
+struct list *wrapper_inputs = 0;
+
 static char *worker_command = 0;
 
 static char *runos_os = 0;
@@ -109,13 +116,26 @@ static struct rmsummary *resources = NULL;
 
 static int64_t factory_timeout = 0;
 
+static char *factory_name = NULL;
+
 struct batch_queue *queue = 0;
 
-//Environment variables to pass along in batch_job_submit
+// Whether workers should use ssl. If using the catalog server and the manager
+// announces it is using SSL, then SSL is used regardless of manual_ssl_option.
+int manual_ssl_option = 0;
+
+//Environment variables to pass along in batch_queue_submit
 struct jx *batch_env = NULL;
 
+//Features to pass along as worker arguments
+struct hash_table *features_table = NULL;
+
 //0 means the container image does not container work_queue_worker binary
 int k8s_worker_image = 0;
+
+// Disable the check for invalid use of AFS with HTCondor.
+static int disable_afs_check = 0;
+
 /*
 In a signal handler, only a limited number of functions are safe to
 invoke, so we construct a string and emit it with a low-level write.
@@ -236,7 +256,24 @@ struct list* do_direct_query( const char *manager_host, int manager_port )
 		return 0;
 	}
 
-	link_putfstring(l,"%s_status\n",stoptime,query_string);
+	if(manual_ssl_option) {
+		if(link_ssl_wrap_connect(l, manager_host) < 1) {
+			fprintf(stderr,"work_queue_factory: could not setup ssl connection.\n");
+			link_close(l);
+			return 0;
+		}
+	}
+
+	if(password) {
+		debug(D_WQ,"authenticating to manager");
+		if(!link_auth_password(l,password,stoptime)) {
+			fprintf(stderr,"work_queue_factory: wrong password for manager.\n");
+			link_close(l);
+			return 0;
+		}
+	}
+
+	link_printf(l,stoptime,"%s_status\n",query_string);
 
 	struct jx *jarray = jx_parse_link(l,stoptime);
 	link_close(l);
@@ -280,7 +317,6 @@ workers are actually connected.
 static int count_workers_needed( struct list *managers_list, int only_not_running )
 {
 	int needed_workers=0;
-	int managers=0;
 	struct jx *j;
 
 	if(!managers_list) {
@@ -297,6 +333,13 @@ static int count_workers_needed( struct list *managers_list, int only_not_runnin
 		const int tr =       jx_lookup_integer(j,"tasks_on_workers");
 		const int tw =       jx_lookup_integer(j,"tasks_waiting");
 		const int tl =       jx_lookup_integer(j,"tasks_left");
+		const int protocol = jx_lookup_integer(j,"protocol");
+
+		if(protocol!=WORK_QUEUE_PROTOCOL_VERSION) {
+			printf("WARNING: skipping manager %s (%s:%d) with protocol version %d\n",project, host, port, protocol);
+			printf("that is not compatible with this factory protocol version %d\n",WORK_QUEUE_PROTOCOL_VERSION);
+			continue;
+		}
 
 		int capacity = manager_workers_capacity(j);
 
@@ -323,7 +366,6 @@ static int count_workers_needed( struct list *managers_list, int only_not_runnin
 
 		debug(D_WQ,"%s %s:%d %s tasks: %d capacity: %d workers needed: %d tasks running: %d",project,host,port,owner,tw+tl+tr,capacity,need,tr);
 		needed_workers += need;
-		managers++;
 	}
 
 	return needed_workers;
@@ -343,19 +385,19 @@ static void set_worker_resources_options( struct batch_queue *queue )
 		}
 	} else {
 		if(resources->cores > -1) {
-			buffer_printf(&b, " --cores=%" PRId64, resources->cores);
+			buffer_printf(&b, " --cores=%s", rmsummary_resource_to_str("cores", resources->cores, 0));
 		}
 
 		if(resources->memory > -1) {
-			buffer_printf(&b, " --memory=%" PRId64, resources->memory);
+			buffer_printf(&b, " --memory=%s", rmsummary_resource_to_str("memory", resources->memory, 0));
 		}
 
 		if(resources->disk > -1) {
-			buffer_printf(&b, " --disk=%" PRId64, resources->disk);
+			buffer_printf(&b, " --disk=%s", rmsummary_resource_to_str("disk", resources->disk, 0));
 		}
 
 		if(resources->gpus > -1) {
-			buffer_printf(&b, " --gpus=%" PRId64, resources->gpus);
+			buffer_printf(&b, " --gpus=%s", rmsummary_resource_to_str("gpus", resources->gpus, 0));
 		}
 	}
 
@@ -365,6 +407,27 @@ static void set_worker_resources_options( struct batch_queue *queue )
 	buffer_free(&b);
 }
 
+/*
+Given a hashtable containing the desired features,
+convert it into a string like "--feature x --feature y"
+to pass to the worker.  The returned string must be freed.
+*/
+
+static char * make_features_string( struct hash_table *features_table )
+{
+	char *str = strdup("");
+
+	char *key;
+	void *value;
+	HASH_TABLE_ITERATE(features_table,key,value) {
+		char * newstr = string_format("%s --feature \"%s\"",str,key);
+		free(str);
+		str = newstr;
+	}
+
+	return str;
+}
+
 static int submit_worker( struct batch_queue *queue )
 {
 	char *cmd;
@@ -376,21 +439,26 @@ static int submit_worker( struct batch_queue *queue )
 		worker = string_format("./%s", worker_command);
 	}
 
+	char *features_string = make_features_string(features_table);
+	
 	if(using_catalog) {
 		cmd = string_format(
-		"%s -M %s -t %d -C '%s' -d all -o worker.log %s %s %s",
+		"%s -M %s -t %d -C '%s' -d all -o worker.log %s %s %s %s %s %s",
 		worker,
 		submission_regex,
 		worker_timeout,
 		catalog_host,
+		factory_name ? string_format("--from-factory \"%s\"", factory_name) : "",
 		password_file ? "-P pwfile" : "",
 		resource_args ? resource_args : "",
+		manual_ssl_option ? "--ssl" : "",
+		features_string,
 		extra_worker_args ? extra_worker_args : ""
 		);
 	}
 	else {
 		cmd = string_format(
-		"%s %s %d -t %d -C '%s' -d all -o worker.log %s %s %s",
+		"%s %s %d -t %d -C '%s' -d all -o worker.log %s %s %s %s %s",
 		worker,
 		manager_host,
 		manager_port,
@@ -398,10 +466,14 @@ static int submit_worker( struct batch_queue *queue )
 		catalog_host,
 		password_file ? "-P pwfile" : "",
 		resource_args ? resource_args : "",
+		manual_ssl_option ? "--ssl" : "",
+		features_string,
 		extra_worker_args ? extra_worker_args : ""
 		);
 	}
 
+	free(features_string);
+	
 	if(wrapper_command) {
 		// Note that we don't use string_wrap_command here,
 		// because the clever quoting interferes with the $$([Target.Memory]) substitution above.
@@ -410,47 +482,44 @@ static int submit_worker( struct batch_queue *queue )
 		cmd = newcmd;
 	}
 
-	char *files = NULL;
-	if(!k8s_worker_image) {
-		files = string_format("work_queue_worker");
+	if(runos_os){
+		char* temp = string_format("%s %s %s",CCTOOLS_RUNOS_PATH,runos_os,cmd);
+		free(cmd);
+		cmd = temp;
+	}
+
+	struct batch_job *task = batch_job_create(queue);
+
+	batch_job_set_command(task,cmd);
+	
+	if(resources) {
+		batch_job_set_resources(task, resources);
+	}
+	
+	if(!runos_os && !k8s_worker_image) {
+		batch_job_add_input_file(task,worker_command,0);
 	} else {
-		files = xxstrdup("");
+		// if runos, then worker comes from vc3_cmd. if k8s, then from the container image.
 	}
 
 	if(password_file) {
-		char *newfiles = string_format("%s,pwfile",files);
-		free(files);
-		files = newfiles;
+		batch_job_add_input_file(task,"pwfile",0);
 	}
 
-	if(wrapper_input) {
-		char *newfiles = string_format("%s,%s",files,wrapper_input);
-		free(files);
-		files = newfiles;
-	}
-
-	if(runos_os){
-		char* vc3_cmd = string_format("./vc3-builder --require cctools-statics -- %s",cmd);
-		char* temp = string_format("python %s %s %s",CCTOOLS_RUNOS_PATH,runos_os,vc3_cmd);
-		free(vc3_cmd);
-		free(cmd);
-		cmd = temp;
-		temp = string_format("%s,%s",files,"vc3-builder");
-		free(files);
-		files = temp;
-	}else{
-		char* temp = string_format("%s,%s",files,worker);
-		free(files);
-		files=temp;
+	const char *item = NULL;
+	LIST_ITERATE(wrapper_inputs,item) {
+		batch_job_add_input_file(task,path_basename(item),0);
 	}
 
+	batch_job_add_output_file(task,"output.log",0);
+	
 	debug(D_WQ,"submitting worker: %s",cmd);
 
+	int status = batch_queue_submit(queue,task);
 
-	int status = batch_job_submit(queue,cmd,files,"output.log",batch_env,resources);
-
+	batch_job_delete(task);
+	
 	free(cmd);
-	free(files);
 	free(worker);
 
 	return status;
@@ -525,7 +594,7 @@ void remove_all_workers( struct batch_queue *queue, struct itable *job_table )
 	itable_firstkey(job_table);
 	while(itable_nextkey(job_table,&jobid,&value)) {
 		debug(D_WQ,"removing job %"PRId64,jobid);
-		batch_job_remove(queue,jobid);
+		batch_queue_remove(queue,jobid);
 	}
 	debug(D_WQ,"%d workers removed.",count);
 
@@ -610,6 +679,8 @@ struct jx *factory_to_jx(struct list *managers, struct list *foremen, int submit
 	if(using_catalog) {
 		jx_insert_string(j, "project_regex",    project_regex);
 		jx_insert_string(j, "submission_regex", submission_regex);
+		jx_insert_integer(j, "max_workers", workers_max);
+		if (factory_name) jx_insert_string(j, "factory_name", factory_name);
 	}
 
 	int to_connect = submitted - connected;
@@ -649,6 +720,12 @@ struct jx *factory_to_jx(struct list *managers, struct list *foremen, int submit
 	}
 	jx_insert(j, jx_string("foremen"), fs);
 
+	char username[USERNAME_MAX];
+	if(username_get(username)) {
+		jx_insert_string(j,"owner",username);
+	}
+	jx_insert_string(j,"version",CCTOOLS_VERSION);
+	
 	return j;
 }
 
@@ -666,10 +743,18 @@ void delete_projects_list(struct list *l)
 #define assign_new_value(new_var, old_var, option, type_c, type_json, field) \
 	type_c new_var = old_var;\
 	{\
-		struct jx *jv = jx_lookup(J,#option); \
+		struct jx *jv = jx_eval(jx_lookup(J, #option), J); \
 		if(jv) {\
-			if(jv->type==type_json) {\
+			if (jv->type == JX_DOUBLE) {\
+				int v = ceil(jv->u.double_value);\
+				jx_delete(jv);\
+				jv = jx_integer(v);\
+			}\
+			if (jv->type == type_json) {\
 				new_var = jv->u.field;\
+			} else if (jv->type == JX_ERROR) {\
+				debug(D_NOTICE, "%s", jv->u.err->u.string_value);\
+				error_found = 1;\
 			} else {\
 				debug(D_NOTICE, #option " has not a valid value.");\
 				error_found = 1;\
@@ -715,7 +800,9 @@ int read_config_file(const char *config_file) {
 
 	assign_new_value(new_factory_timeout_option, factory_timeout, factory-timeout, int, JX_INTEGER, integer_value)
 
-	assign_new_value(new_tasks_per_worker, tasks_per_worker, tasks-per-worker, double, JX_INTEGER, integer_value)
+	assign_new_value(new_tasks_per_worker, tasks_per_worker, tasks-per-worker, int, JX_INTEGER, integer_value)
+
+	assign_new_value(new_factory_name, factory_name, factory-name, const char *, JX_STRING, string_value)
 
 	/* first try with old master option, then with manager */
 	assign_new_value(new_project_regex_old_opt, project_regex, master-name, const char *, JX_STRING, string_value)
@@ -737,21 +824,23 @@ int read_config_file(const char *config_file) {
 		}
 	}
 
-	if(new_workers_min > new_workers_max) {
-		debug(D_NOTICE, "%s: min workers (%d) is greater than max workers (%d)\n", config_file, new_workers_min, new_workers_max);
-		error_found = 1;
-	}
-
 	if(new_workers_min < 0) {
 		debug(D_NOTICE, "%s: min workers (%d) is less than zero.\n", config_file, new_workers_min);
 		error_found = 1;
 	}
 
-	if(new_workers_max < 0) {
-		debug(D_NOTICE, "%s: max workers (%d) is less than zero.\n", config_file, new_workers_max);
+	if(new_workers_max < 1) {
+		debug(D_NOTICE, "%s: max workers (%d) is less than one.\n", config_file, new_workers_max);
 		error_found = 1;
 	}
 
+	if(new_workers_min > new_workers_max) {
+		debug(D_NOTICE, "%s: min workers (%d) is greater than max workers (%d)\n", config_file, new_workers_min, new_workers_max);
+		debug(D_NOTICE, "setting min workers and max workers to %d\n", new_workers_max);
+		new_workers_min = new_workers_max;
+	}
+
+
 	if(new_factory_timeout_option < 0) {
 		debug(D_NOTICE, "%s: factory timeout (%d) is less than zero.\n", config_file, new_factory_timeout_option);
 		error_found = 1;
@@ -799,6 +888,11 @@ int read_config_file(const char *config_file) {
 		condor_requirements = xxstrdup(new_condor_requirements);
 	}
 
+	if(new_factory_name && new_factory_name != factory_name) {
+		free(factory_name);
+		factory_name = xxstrdup(new_factory_name);
+	}
+
 	last_time_modified = new_time_modified;
 	fprintf(stdout, "Configuration file '%s' has been loaded.", config_file);
 
@@ -810,9 +904,13 @@ int read_config_file(const char *config_file) {
 	fprintf(stdout, "min-workers: %d\n", workers_min);
 	fprintf(stdout, "workers-per-cycle: %d\n", workers_per_cycle);
 
-	fprintf(stdout, "tasks-per-worker: %" PRId64 "\n", tasks_per_worker > 0 ? tasks_per_worker : (resources->cores > 0 ? resources->cores : 1));
+	fprintf(stdout, "tasks-per-worker: %d\n", tasks_per_worker > 0 ? tasks_per_worker : (resources->cores > 0 ? (int) resources->cores : 1));
 	fprintf(stdout, "timeout: %d s\n", worker_timeout);
-	fprintf(stdout, "cores: %" PRId64 "\n", resources->cores > 0 ? resources->cores : 1);
+	fprintf(stdout, "cores: %s\n", rmsummary_resource_to_str("cores", resources->cores > 0 ? resources->cores : 1, 0));
+
+	if(factory_name) {
+		fprintf(stdout, "factory_name: %s\n", factory_name);
+	}
 
 	if(condor_requirements) {
 		fprintf(stdout, "condor-requirements: %s\n", condor_requirements);
@@ -823,15 +921,15 @@ int read_config_file(const char *config_file) {
 	}
 
 	if(resources->memory > -1) {
-		fprintf(stdout, "memory: %" PRId64 " MB\n", resources->memory);
+		fprintf(stdout, "memory: %s\n", rmsummary_resource_to_str("memory", resources->memory, 1));
 	}
 
 	if(resources->disk > -1) {
-		fprintf(stdout, "disk: %" PRId64 " MB\n", resources->disk);
+		fprintf(stdout, "disk: %s\n", rmsummary_resource_to_str("disk", resources->disk, 1));
 	}
 
 	if(resources->gpus > -1) {
-		fprintf(stdout, "gpus: %" PRId64 "\n", resources->gpus);
+		fprintf(stdout, "gpus: %s\n", rmsummary_resource_to_str("gpus", resources->gpus, 0));
 	}
 
 	if(extra_worker_args) {
@@ -862,6 +960,11 @@ static void mainloop( struct batch_queue *queue )
 	int64_t factory_timeout_start = time(0);
 
 	while(!abort_flag) {
+		if (initial_ppid != 0 && getppid() != initial_ppid) {
+			printf("parent process exited, shutting down\n");
+			abort_flag = 1;
+			break;
+		}
 
 		if(config_file && !read_config_file(config_file)) {
 			debug(D_NOTICE, "Error re-reading '%s'. Using previous values.", config_file);
@@ -944,22 +1047,21 @@ static void mainloop( struct batch_queue *queue )
 		int workers_waiting_to_connect = workers_submitted - workers_connected;
 
 		if(workers_waiting_to_connect < 0) {
-			debug(D_WQ,"%d workers already connected from other sources", -workers_waiting_to_connect);
+			debug(D_WQ,"at least %d workers have already connected from other sources", -workers_waiting_to_connect);
+			new_workers_needed -= abs(workers_waiting_to_connect);
+
+			// this factory has no workers_waiting_to_connect
+			workers_waiting_to_connect = 0;
 		}
 
 		if(workers_waiting_to_connect > 0) {
 			debug(D_WQ,"waiting for %d previously submitted workers to connect", workers_waiting_to_connect);
 		}
 
-		//abs here because:
-		//if +, we are waiting for workers to connect, thus we don't need to submit as many new ones.
-		//if -, workers connected from other sources, thus we don't need to submit as many new ones.
-		new_workers_needed = new_workers_needed - abs(workers_waiting_to_connect);
-
-		// Always apply workers_per_cycle at the very end
-		if(workers_per_cycle > 0 && new_workers_needed > workers_per_cycle) {
+		// Apply workers_per_cycle. Never have more than workers_per_cycle waiting to connect.
+		if(workers_per_cycle > 0 && (new_workers_needed + workers_waiting_to_connect) > workers_per_cycle) {
 			debug(D_WQ,"applying maximum workers per cycle of %d",workers_per_cycle);
-			new_workers_needed = workers_per_cycle;
+			new_workers_needed = MAX(0, workers_per_cycle - workers_waiting_to_connect);
 		}
 
 		debug(D_WQ,"workers needed: %d",    workers_needed);
@@ -970,7 +1072,7 @@ static void mainloop( struct batch_queue *queue )
 
 		char *update_str = jx_print_string(j);
 		debug(D_WQ, "Sending status to the catalog server(s) at %s ...", catalog_host);
-		catalog_query_send_update(catalog_host, update_str);
+		catalog_query_send_update(catalog_host,update_str,0);
 		print_stats(j);
 		free(update_str);
 		jx_delete(j);
@@ -991,8 +1093,8 @@ static void mainloop( struct batch_queue *queue )
 
 		while(1) {
 			struct batch_job_info info;
-			batch_job_id_t jobid;
-			jobid = batch_job_wait_timeout(queue,&info,stoptime);
+			batch_queue_id_t jobid;
+			jobid = batch_queue_wait_timeout(queue,&info,stoptime);
 			if(jobid>0) {
 				if(itable_lookup(job_table,jobid)) {
 					itable_remove(job_table,jobid);
@@ -1009,7 +1111,14 @@ static void mainloop( struct batch_queue *queue )
 		delete_projects_list(managers_list);
 		delete_projects_list(foremen_list);
 
-		sleep(factory_period);
+		int sleep_seconds = 0;
+		while(sleep_seconds < factory_period) {
+			if(abort_flag) {
+				break;
+			}
+			sleep_seconds += 1;
+			sleep(1);
+		}
 	}
 
 	printf("removing %d workers...\n",itable_size(job_table));
@@ -1018,56 +1127,93 @@ static void mainloop( struct batch_queue *queue )
 	itable_delete(job_table);
 }
 
+/* Add a wrapper command around the worker executable. */
+/* Note that multiple wrappers can be nested. */
+
+void add_wrapper_command( const char *cmd )
+{
+	if(!wrapper_command) {
+		wrapper_command = strdup(cmd);
+	} else {
+		char *tmp = string_format("%s %s",cmd,wrapper_command);
+		free(wrapper_command);
+		wrapper_command = tmp;
+	}
+}
+
+/* Add an additional input file to be consumed by the wrapper. */
+
+void add_wrapper_input( const char *filename )
+{
+	list_push_tail(wrapper_inputs, xxstrdup(filename));
+}
+
 static void show_help(const char *cmd)
 {
-	printf("Use: work_queue_factory [options] <managerhost> <port>\nor\n     work_queue_factory [options] -M projectname\n");
-	printf("where options are:\n");
-	printf(" %-30s Project name of managers to serve, can be a regular expression.\n", "-M,-N,--manager-name=<project>");\
-	printf(" %-30s Foremen to serve, can be a regular expression.\n", "-F,--foremen-name=<project>");
-	printf(" %-30s Catalog server to query for managers (default: %s:%d).\n", "--catalog=<host:port>",CATALOG_HOST,CATALOG_PORT);
-	printf(" %-30s Batch system type (required). One of:\n", "-T,--batch-type=<type>");
-	printf(" %-30s %s\n","",batch_queue_type_string());
-	printf(" %-30s Add these options to all batch submit files.\n", "-B,--batch-options=<options>");
-	printf(" %-30s Password file for workers to authenticate to manager.\n","-P,--password");
+	printf("Use: work_queue_factory [options] <managerhost> <port>\n");
+	printf("Or:  work_queue_factory [options] -M projectname\n");
+	printf("\n");
+	/*------------------------------------------------------------*/
+	printf("General options:\n");
+	printf(" %-30s Batch system type (required). One of: %s\n", "-T,--batch-type=<type>",batch_queue_type_string());
 	printf(" %-30s Use configuration file <file>.\n","-C,--config-file=<file>");
-	printf(" %-30s Minimum workers running (default=%d).\n", "-w,--min-workers", workers_min);
-	printf(" %-30s Maximum workers running (default=%d).\n", "-W,--max-workers", workers_max);
-	printf(" %-30s Maximum number of new workers per %d s (less than 1 disables limit, default=%d).\n", "--workers-per-cycle", factory_period, workers_per_cycle);
-	printf(" %-30s Average tasks per worker (default=one task per core).\n", "--tasks-per-worker");
-	printf(" %-30s Workers abort after this amount of idle time (default=%d).\n", "-t,--timeout=<time>",worker_timeout);
-	printf(" %-30s Environment variable that should be added to the worker (May be specified multiple times).\n", "--env=<variable=value>");
-	printf(" %-30s Extra options that should be added to the worker.\n", "-E,--extra-options=<options>");
-	printf(" %-30s Set the number of cores requested per worker.\n", "--cores=<n>");
-	printf(" %-30s Set the number of GPUs requested per worker.\n", "--gpus=<n>");
-	printf(" %-30s Set the amount of memory (in MB) requested per worker.\n", "--memory=<mb>           ");
-	printf(" %-30s Set the amount of disk (in MB) requested per worker.\n", "--disk=<mb>");
-	printf(" %-30s Automatically size a worker to an available slot (Condor, Mesos, and Kubernetes).\n", "--autosize");
-	printf(" %-30s Set requirements for the workers as Condor jobs. May be specified several times with expresions and-ed together (Condor only).\n", "--condor-requirements");
-	printf(" %-30s Exit after no manager has been seen in <n> seconds.\n", "--factory-timeout");
-	printf(" %-30s Use this scratch dir for temporary files (default is /tmp/wq-factory-$uid).\n","-S,--scratch-dir");
-	printf(" %-30s Use worker capacity reported by managers.\n","-c,--capacity");
+	printf(" %-30s Project name of managers to server, can be regex\n","-M,-N,--manager-name=<project>");\
+	printf(" %-30s Foremen to serve, can be a regular expression.\n", "-F,--foremen-name=<project>");
+	printf(" %-30s Catalog server to query for managers.\n", "--catalog=<host:port>");
+	printf(" %-30s Password file for workers to authenticate.\n","-P,--password");
+	printf(" %-30s Use this scratch dir for factory.\n","-S,--scratch-dir");
+	printf(" %-30s (default: /tmp/wq-factory-$uid).\n","");
+	printf(" %-30s Force factory to run itself as a manager.\n","--run-factory-as-manager");
+	printf(" %-30s Exit if parent process dies.\n", "--parent-death");
 	printf(" %-30s Enable debugging for this subsystem.\n", "-d,--debug=<subsystem>");
-	printf(" %-30s Specify Amazon config file (for use with -T amazon).\n", "--amazon-config");
-	printf(" %-30s Wrap factory with this command prefix.\n","--wrapper");
-	printf(" %-30s Add this input file needed by the wrapper.\n","--wrapper-input");
-	printf(" %-30s Specify the host name to mesos manager node (for use with -T mesos).\n", "--mesos-master");
-	printf(" %-30s Specify path to mesos python library (for use with -T mesos).\n", "--mesos-path");
-	printf(" %-30s Specify the linking libraries for running mesos (for use with -T mesos).\n", "--mesos-preload");
-	printf(" %-30s Specify the container image for using Kubernetes (for use with -T k8s).\n", "--k8s-image");
-	printf(" %-30s Specify the container image that contains work_queue_worker availabe for using Kubernetes (for use with -T k8s).\n", "--k8s-worker-image");
-	printf(" %-30s Send debugging to this file (can also be :stderr, or :stdout).\n", "-o,--debug-file=<file>");
-	printf(" %-30s Specify the size of the debug file (must use with -o option).\n", "-O,--debug-file-size=<mb>");
-	printf(" %-30s Specify the binary to use for the worker (relative or hard path). It should accept the same arguments as the default work_queue_worker.\n", "--worker-binary=<file>");
-	printf(" %-30s Will make a best attempt to ensure the worker will execute in the specified OS environment, regardless of the underlying OS.\n","--runos=<img>");
-	printf(" %-30s Force factory to run itself as a work queue manager.\n","--run-factory-as-manager");
+	printf(" %-30s Send debugging to this file.\n", "-o,--debug-file=<file>");
+	printf(" %-30s Specify the size of the debug file.\n", "-O,--debug-file-size=<mb>");
+	printf(" %-30s Workers should use SSL to connect to managers. (Not needed if project names.)", "--ssl");
 	printf(" %-30s Show the version string.\n", "-v,--version");
 	printf(" %-30s Show this screen.\n", "-h,--help");
+
+	/*------------------------------------------------------------*/
+	printf("\nConcurrency control options:\n");
+	printf(" %-30s Minimum workers running (default=%d).\n", "-w,--min-workers", workers_min);
+	printf(" %-30s Maximum workers running (default=%d).\n", "-W,--max-workers", workers_max);
+	printf(" %-30s Max number of new workers per %ds (default=%d)\n", "--workers-per-cycle", factory_period, workers_per_cycle);
+	printf(" %-30s Workers abort after idle time (default=%d).\n", "-t,--timeout=<time>",worker_timeout);
+	printf(" %-30s Exit after no manager seen in <n> seconds.\n", "--factory-timeout");
+	printf(" %-30s Average tasks per worker (default=one per core).\n", "--tasks-per-worker");
+	printf(" %-30s Use worker capacity reported by managers.\n","-c,--capacity");
+
+	printf("\nResource management options:\n");
+	printf(" %-30s Set the number of cores requested per worker.\n", "--cores=<n>");
+	printf(" %-30s Set the number of GPUs requested per worker.\n", "--gpus=<n>");
+	printf(" %-30s Set the amount of memory (in MB) per worker.\n", "--memory=<mb>           ");
+	printf(" %-30s Set the amount of disk (in MB) per worker.\n", "--disk=<mb>");
+	printf(" %-30s Add a custom feature to each worker.\n", "--feature=<name>");
+	printf(" %-30s Autosize worker to slot (Condor, Mesos, K8S).\n", "--autosize");
+
+	printf("\nWorker environment options:\n");
+	printf(" %-30s Environment variable to add to worker.\n", "--env=<variable=value>");
+	printf(" %-30s Extra options to give to worker.\n", "-E,--extra-options=<options>");
+	printf(" %-30s Alternate binary instead of work_queue_worker.\n", "--worker-binary=<file>");
+	printf(" %-30s Wrap factory with this command prefix.\n","--wrapper");
+	printf(" %-30s Add this input file needed by the wrapper.\n","--wrapper-input");
+	printf(" %-30s Use runos tool to create environment (ND only).\n","--runos=<img>");
+	printf(" %-30s Run each worker inside this poncho environment.\n","--poncho-env=<file.tar.gz>");
+
+	printf("\nOptions specific to batch systems:\n");
+	printf(" %-30s Generic batch system options.\n", "-B,--batch-options=<options>");
+	printf(" %-30s Disable check for use of AFS with HTCondor.\n", "--disable-afs-check");
+	printf(" %-30s Specify Amazon config file.\n", "--amazon-config");
+	printf(" %-30s Set requirements for the workers as Condor jobs.\n", "--condor-requirements");
+	printf(" %-30s Container image for Kubernetes.\n", "--k8s-image");
+	printf(" %-30s Container image with worker for Kubernetes.\n", "--k8s-worker-image");
+
 }
 
 enum{   LONG_OPT_CORES = 255,
 		LONG_OPT_MEMORY, 
 		LONG_OPT_DISK, 
 		LONG_OPT_GPUS, 
+	        LONG_OPT_FEATURE,
 		LONG_OPT_TASKS_PER_WORKER, 
 		LONG_OPT_CONF_FILE, 
 		LONG_OPT_AMAZON_CONFIG, 
@@ -1078,15 +1224,17 @@ enum{   LONG_OPT_CORES = 255,
 		LONG_OPT_WRAPPER, 
 		LONG_OPT_WRAPPER_INPUT,
 		LONG_OPT_WORKER_BINARY,
-		LONG_OPT_MESOS_MANAGER, 
-		LONG_OPT_MESOS_PATH,
-		LONG_OPT_MESOS_PRELOAD,
 		LONG_OPT_K8S_IMAGE,
 		LONG_OPT_K8S_WORKER_IMAGE,
 		LONG_OPT_CATALOG,
 		LONG_OPT_ENVIRONMENT_VARIABLE,
 		LONG_OPT_RUN_AS_MANAGER,
 		LONG_OPT_RUN_OS,
+		LONG_OPT_PARENT_DEATH,
+		LONG_OPT_PONCHO_ENV,
+		LONG_OPT_USE_SSL,
+		LONG_OPT_FACTORY_NAME,
+		LONG_OPT_DISABLE_AFS_CHECK,
 	};
 
 static const struct option long_options[] = {
@@ -1102,10 +1250,12 @@ static const struct option long_options[] = {
 	{"debug", required_argument, 0, 'd'},
 	{"debug-file", required_argument, 0, 'o'},
 	{"debug-file-size", required_argument, 0, 'O'},
+	{"disable-afs-check", no_argument, 0, LONG_OPT_DISABLE_AFS_CHECK },
 	{"disk",   required_argument,  0,  LONG_OPT_DISK},
 	{"env", required_argument, 0, LONG_OPT_ENVIRONMENT_VARIABLE},
 	{"extra-options", required_argument, 0, 'E'},
 	{"factory-timeout", required_argument, 0, LONG_OPT_FACTORY_TIMEOUT},
+	{"feature", required_argument, 0, LONG_OPT_FEATURE},
 	{"foremen-name", required_argument, 0, 'F'},
 	{"gpus",   required_argument,  0,  LONG_OPT_GPUS},
 	{"help", no_argument, 0, 'h'},
@@ -1115,11 +1265,12 @@ static const struct option long_options[] = {
 	{"master-name", required_argument, 0, 'M'},
 	{"max-workers", required_argument, 0, 'W'},
 	{"memory", required_argument,  0,  LONG_OPT_MEMORY},
-	{"mesos-master", required_argument, 0, LONG_OPT_MESOS_MANAGER},
-	{"mesos-path", required_argument, 0, LONG_OPT_MESOS_PATH},
-	{"mesos-preload", required_argument, 0, LONG_OPT_MESOS_PRELOAD},
 	{"min-workers", required_argument, 0, 'w'},
+	{"parent-death", no_argument, 0, LONG_OPT_PARENT_DEATH},
 	{"password", required_argument, 0, 'P'},
+	{"poncho-env", required_argument, 0, LONG_OPT_PONCHO_ENV},
+	{"python-env", required_argument, 0, LONG_OPT_PONCHO_ENV}, // backwards compatibility
+	{"python-package", required_argument, 0, LONG_OPT_PONCHO_ENV}, // backwards compatibility
 	{"run-factory-as-manager", no_argument, 0, LONG_OPT_RUN_AS_MANAGER},
 	{"runos", required_argument, 0, LONG_OPT_RUN_OS},
 	{"scratch-dir", required_argument, 0, 'S' },
@@ -1129,18 +1280,18 @@ static const struct option long_options[] = {
 	{"worker-binary", required_argument, 0, LONG_OPT_WORKER_BINARY},
 	{"workers-per-cycle", required_argument, 0, LONG_OPT_WORKERS_PER_CYCLE},
 	{"wrapper",required_argument, 0, LONG_OPT_WRAPPER},
-	{"wrapper-input",required_argument, 0, LONG_OPT_WRAPPER_INPUT},	
+	{"wrapper-input",required_argument, 0, LONG_OPT_WRAPPER_INPUT},
+	{"ssl",no_argument, 0, LONG_OPT_USE_SSL},
+	{"factory-name",required_argument, 0, LONG_OPT_FACTORY_NAME},
 	{0,0,0,0}
 };
 
 
 int main(int argc, char *argv[])
 {
-	char *mesos_manager = NULL;
-	char *mesos_path = NULL;
-	char *mesos_preload = NULL;
 	char *k8s_image = NULL;
-	struct jx *wrapper_inputs = jx_array(NULL);
+
+	wrapper_inputs = list_create();
 
 	// --run-factory-as-manager and -Twq should be used together.
 	int run_as_manager = 0;
@@ -1150,6 +1301,7 @@ int main(int argc, char *argv[])
 	char *env = NULL;
 	char *val = NULL;
 	batch_env = jx_object(NULL);
+	features_table = hash_table_create(0,0);
 
 	batch_queue_type_t batch_queue_type = BATCH_QUEUE_TYPE_UNKNOWN;
 
@@ -1235,6 +1387,9 @@ int main(int argc, char *argv[])
 			case LONG_OPT_GPUS:
 				resources->gpus = atoi(optarg);
 				break;
+			case LONG_OPT_FEATURE:
+				hash_table_insert(features_table,optarg,"true");
+				break;
 			case LONG_OPT_AUTOSIZE:
 				autosize = 1;
 				break;
@@ -1250,23 +1405,34 @@ int main(int argc, char *argv[])
 					condor_requirements = string_format("(%s)", optarg);
 				}
 				break;
+			case LONG_OPT_PONCHO_ENV:
+				{
+				// --poncho-env X is the equivalent of --wrapper "poncho_package_run X" --wrapper-input X
+				char *fullpath = path_which("poncho_package_run");
+				if(!fullpath) {
+					fprintf(stderr,"work_queue_factory: could not find poncho_package_run in PATH");
+					return 1;
+				}
+				add_wrapper_input(fullpath);
+				add_wrapper_input(optarg);
+				add_wrapper_command( string_format("./%s -e %s ",path_basename(fullpath), path_basename(optarg)));
+				break;
+				}
 			case LONG_OPT_WRAPPER:
-				wrapper_command = optarg;
+				add_wrapper_command(optarg);
 				break;
 			case LONG_OPT_WRAPPER_INPUT:
-				if(!wrapper_input) {
-					wrapper_input = strdup(optarg);
-				} else {
-					wrapper_input = string_format("%s,%s",wrapper_input,optarg);
-				}
-				struct jx *file = jx_string(optarg);
-				jx_array_append(wrapper_inputs, file);
+				add_wrapper_input(optarg);
 				break;
 			case LONG_OPT_WORKER_BINARY:
 				worker_command = xxstrdup(optarg);
 				break;
 			case 'P':
 				password_file = optarg;
+				if(copy_file_to_buffer(optarg, &password, NULL) < 0) {
+					fprintf(stderr,"work_queue_factory: couldn't load password from %s: %s\n",optarg,strerror(errno));
+					exit(EXIT_FAILURE);
+				}
 				break;
 			case 'S':
 				scratch_dir = optarg;
@@ -1275,7 +1441,10 @@ int main(int argc, char *argv[])
 				consider_capacity = 1;
 				break;
 			case 'd':
-				debug_flags_set(optarg);
+				if (!debug_flags_set(optarg)) {
+					fprintf(stderr, "Unknown debug flag: %s\n", optarg);
+					exit(EXIT_FAILURE);
+				}
 				break;
 			case 'o':
 				debug_config_file(optarg);
@@ -1289,15 +1458,6 @@ int main(int argc, char *argv[])
 			case 'h':
 				show_help(argv[0]);
 				exit(EXIT_SUCCESS);
-			case LONG_OPT_MESOS_MANAGER:
-				mesos_manager = xxstrdup(optarg);
-				break;
-			case LONG_OPT_MESOS_PATH:
-				mesos_path = xxstrdup(optarg);
-				break;
-			case LONG_OPT_MESOS_PRELOAD:
-				mesos_preload = xxstrdup(optarg);
-				break;
 			case LONG_OPT_K8S_IMAGE:
 				k8s_image = xxstrdup(optarg);
 				break;
@@ -1314,6 +1474,18 @@ int main(int argc, char *argv[])
 			case LONG_OPT_RUN_OS:
 				runos_os = xxstrdup(optarg);
 				break;
+			case LONG_OPT_PARENT_DEATH:
+				initial_ppid = getppid();
+				break;
+			case LONG_OPT_USE_SSL:
+				manual_ssl_option=1;
+				break;
+			case LONG_OPT_FACTORY_NAME:
+				factory_name = xxstrdup(optarg);
+				break;
+			case LONG_OPT_DISABLE_AFS_CHECK:
+				disable_afs_check = 1;
+				break;
 			default:
 				show_help(argv[0]);
 				return EXIT_FAILURE;
@@ -1358,7 +1530,6 @@ int main(int argc, char *argv[])
 
 	if(!(manager_host || config_file || project_regex)) {
 		fprintf(stderr,"work_queue_factory: You must either give a project name with the -M option or manager-name option with a configuration file, or give the manager's host and port.\n");
-		show_help(argv[0]);
 		exit(1);
 	}
 
@@ -1398,15 +1569,22 @@ int main(int argc, char *argv[])
 	/*
 	Careful here: most of the supported batch systems expect
 	that jobs are submitting from a single shared filesystem.
-	Changing to /tmp only works in the case of Condor.
+	In general, we will put log files into a subdir of the
+	current working directory, with a unique name to separate
+	factory instances.
+
+	However, HTCondor has two constraints:
+	1 - Recent versions of HTCondor insist upon the user log
+	being written to a file under $HOME, for reasons unknown.
+	It will emit errors at submit time if this happens.
+
+	2 - Condor cannot easily deal with files submitted from
+	an AFS home directory, without making things world writeable.
+	We will complain about that here.
 	*/
 
 	if(!scratch_dir) {
-		if(batch_queue_type==BATCH_QUEUE_TYPE_CONDOR) {
-			scratch_dir = string_format("/tmp/wq-factory-%d",getuid());
-		} else {
-			scratch_dir = string_format("wq-factory-%d",getuid());
-		}
+		scratch_dir = string_format("wq-factory-%d",getuid());
 	}
 
 	if(!create_dir(scratch_dir,0777)) {
@@ -1414,30 +1592,48 @@ int main(int argc, char *argv[])
 		return 1;
 	}
 
-	if(wrapper_input) {
-		struct jx *item;
-		for(void *i = NULL; (item = jx_iterate_array(wrapper_inputs, &i));) {
-			const char *value = item->u.string_value;
-			const char *file_at_scratch_dir = string_format("%s/%s", scratch_dir, path_basename(value));
-			int result = copy_direntry(value, file_at_scratch_dir); 
-			if(result < 0) {
-				debug(D_NOTICE, "Cannot copy wrapper input file %s to factory scratch directory", value);
-			}
+	if(batch_queue_type==BATCH_QUEUE_TYPE_CONDOR && !disable_afs_check) {
+		char *absolute_scratch_dir = realpath(scratch_dir,0);
+		if(!absolute_scratch_dir) {
+			fprintf(stderr,"work_queue_factory: couldn't get full path of %s: %s\n",scratch_dir,strerror(errno));
+			return 1;
 		}
+
+		if(!strncmp(absolute_scratch_dir,"/afs",4)) {
+			fprintf(stderr,"work_queue_factory: The scratch directory is '%s'\n", absolute_scratch_dir);
+			fprintf(stderr,"This won't work because Condor is not able to write to files in AFS.\n");
+			fprintf(stderr,"Please use --scratch-dir to choose a different scratch directory.\n");
+			return 1;
+		}
+	}
+		
+	const char *item = NULL;
+	list_first_item(wrapper_inputs);
+	while((item = list_next_item(wrapper_inputs))) {
+		char *file_at_scratch_dir = string_format("%s/%s", scratch_dir, path_basename(item));
+		int64_t result = copy_file_to_file(item, file_at_scratch_dir);
+		if(result < 0) {
+			fprintf(stderr,"work_queue_factory: Cannot copy wrapper input file %s to factory scratch directory %s:\n", item, file_at_scratch_dir);
+			fprintf(stderr,"%s\n", strerror(errno));
+			exit(EXIT_FAILURE);
+		}
+		free(file_at_scratch_dir);
 	}
 
 	char* cmd;
-	if(worker_command != NULL){
+	if(runos_os) {
+		worker_command = xxstrdup("work_queue_worker");
+	} else if(worker_command != NULL) {
 		cmd = string_format("cp '%s' '%s'",worker_command,scratch_dir);
 		if(system(cmd)){
-			fprintf(stderr, "work_queue_factory: Could not Access specified worker binary.\n");
+			fprintf(stderr,"work_queue_factory: Could not Access specified worker binary.\n");
 			exit(EXIT_FAILURE);
 		}
 		free(cmd);
 		char *tmp = xxstrdup(path_basename(worker_command));
 		free(worker_command);
 		worker_command = tmp;
-	}else{
+	} else {
 		worker_command = xxstrdup("work_queue_worker");
 		cmd = string_format("cp \"$(which %s)\" '%s'",worker_command,scratch_dir);
 		if (system(cmd)) {
@@ -1447,15 +1643,6 @@ int main(int argc, char *argv[])
 		free(cmd);
 	}
 
-	if(runos_os) {
-		cmd = string_format("cp '%s' '%s'",  CCTOOLS_VC3_BUILDER_PATH, scratch_dir);
-		int k = system(cmd);
-		if (k) {
-			fprintf(stderr, "can't copy vc3-builder! Please make sure it's at `%s`. Error code: %i\n",  CCTOOLS_VC3_BUILDER_PATH, k);
-			exit(EXIT_FAILURE);
-		}
-	}
-
 	if(password_file) {
 		cmd = string_format("cp %s %s/pwfile",password_file,scratch_dir);
 		system(cmd);
@@ -1472,7 +1659,7 @@ int main(int argc, char *argv[])
 	signal(SIGTERM, handle_abort);
 	signal(SIGHUP, ignore_signal);
 
-	queue = batch_queue_create(batch_queue_type);
+	queue = batch_queue_create(batch_queue_type,0,0);
 	if(!queue) {
 		fprintf(stderr,"work_queue_factory: couldn't establish queue type %s",batch_queue_type_to_string(batch_queue_type));
 		return 1;
@@ -1492,30 +1679,15 @@ int main(int argc, char *argv[])
 		batch_queue_set_option(queue, "condor-requirements", condor_requirements);
 	}
 
-	if(batch_queue_type == BATCH_QUEUE_TYPE_MESOS) {
-		batch_queue_set_option(queue, "mesos-path", mesos_path);
-		batch_queue_set_option(queue, "mesos-master", mesos_manager);
-		batch_queue_set_option(queue, "mesos-preload", mesos_preload);
-		batch_queue_set_logfile(queue, "work_queue_factory.mesoslog");
-	}
-	
 	if(batch_queue_type == BATCH_QUEUE_TYPE_K8S) {
 		batch_queue_set_option(queue, "k8s-image", k8s_image);
 	}
 
 	mainloop( queue );
 
-	if(batch_queue_type == BATCH_QUEUE_TYPE_MESOS) {
-
-		batch_queue_set_int_option(queue, "batch-queue-abort-flag", (int)abort_flag);
-		batch_queue_set_int_option(queue, "batch-queue-failed-flag", 0);
-
-	}
-
-	jx_delete(wrapper_inputs);
 	batch_queue_delete(queue);
 
 	return 0;
 }
 
-/* vim: set noexpandtab tabstop=4: */
+/* vim: set noexpandtab tabstop=8: */
