@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2008- The University of Notre Dame
+Copyright (C) 2022 The University of Notre Dame
 This software is distributed under the GNU General Public License.
 See the file COPYING for details.
 */
@@ -11,7 +11,6 @@ See the file COPYING for details.
 #include "chirp_fs_chirp.h"
 #include "chirp_fs_hdfs.h"
 #include "chirp_fs_local.h"
-#include "chirp_fs_confuga.h"
 
 #include "debug.h"
 #include "macros.h"
@@ -64,8 +63,6 @@ struct chirp_filesystem *cfs_lookup(const char *url)
 		return &chirp_fs_chirp;
 	} else if(strprfx(url, "hdfs://")) {
 		return &chirp_fs_hdfs;
-	} else if(strprfx(url, "confuga://")) {
-		return &chirp_fs_confuga;
 	} else {
 		/* always interpret as a local url */
 		return &chirp_fs_local;
@@ -81,11 +78,6 @@ void cfs_normalize(char url[CHIRP_PATH_MAX])
 		return;
 	} else if(strprfx(url, "hdfs:")) {
 		return;
-	} else if(pattern_match(url, "^confuga://([^?]*)(.*)", &root, &rest) >= 0) {
-		char absolute[PATH_MAX];
-		path_absolute(root, absolute, 0);
-		debug(D_CHIRP, "normalizing url `%s' as `confuga://%s%s'", url, absolute, rest);
-		string_nformat(url, CHIRP_PATH_MAX, "confuga://%s%s", absolute, rest);
 	} else {
 		char absolute[PATH_MAX];
 		if(strprfx(url, "file:") || strprfx(url, "local:"))
@@ -764,17 +756,17 @@ static int search_directory(const char *subject, const char *const base, char fu
 					if(metadata) {
 						/* A match was found, but the matched file couldn't be statted. Generate a result and an error. */
 						if(entry->lstatus == -1) {
-							link_putfstring(l, "0:%s::\n", stoptime, matched);	// FIXME is this a bug?
-							link_putfstring(l, "%d:%d:%s:\n", stoptime, errno, CHIRP_SEARCH_ERR_STAT, matched);
+							link_printf(l, stoptime, "0:%s::\n", matched);	// FIXME is this a bug?
+							link_printf(l, stoptime, "%d:%d:%s:\n", errno, CHIRP_SEARCH_ERR_STAT, matched);
 						} else {
 							BUFFER_STACK_ABORT(B, 4096)
 							chirp_stat_encode(B, &entry->info);
-							link_putfstring(l, "0:%s:%s:\n", stoptime, matched, buffer_tostring(B));
+							link_printf(l, stoptime, "0:%s:%s:\n", matched, buffer_tostring(B));
 							if(stopatfirst)
 								return 1;
 						}
 					} else {
-						link_putfstring(l, "0:%s::\n", stoptime, matched);
+						link_printf(l, stoptime, "0:%s::\n", matched);
 						if(stopatfirst)
 							return 1;
 					}
@@ -790,7 +782,7 @@ static int search_directory(const char *subject, const char *const base, char fu
 							return result;
 					}
 				} else {
-					link_putfstring(l, "%d:%d:%s:\n", stoptime, EPERM, CHIRP_SEARCH_ERR_OPEN, fullpath);
+					link_printf(l, stoptime, "%d:%d:%s:\n", EPERM, CHIRP_SEARCH_ERR_OPEN, fullpath);
 				}
 			}
 			*current = '\0';	/* clear current entry */
@@ -798,14 +790,14 @@ static int search_directory(const char *subject, const char *const base, char fu
 		}
 
 		if(errno)
-			link_putfstring(l, "%d:%d:%s:\n", stoptime, errno, CHIRP_SEARCH_ERR_READ, fullpath);
+			link_printf(l, stoptime, "%d:%d:%s:\n", errno, CHIRP_SEARCH_ERR_READ, fullpath);
 
 		errno = 0;
 		cfs->closedir(dirp);
 		if(errno)
-			link_putfstring(l, "%d:%d:%s:\n", stoptime, errno, CHIRP_SEARCH_ERR_CLOSE, fullpath);
+			link_printf(l, stoptime, "%d:%d:%s:\n", errno, CHIRP_SEARCH_ERR_CLOSE, fullpath);
 	} else {
-		link_putfstring(l, "%d:%d:%s:\n", stoptime, errno, CHIRP_SEARCH_ERR_OPEN, fullpath);
+		link_printf(l, stoptime, "%d:%d:%s:\n", errno, CHIRP_SEARCH_ERR_OPEN, fullpath);
 	}
 
 	return result;
@@ -907,14 +899,4 @@ INT64_T cfs_stub_lremovexattr(const char *path, const char *name)
 	return -1;
 }
 
-int cfs_stub_job_dbinit(sqlite3 * db)
-{
-	return ENOSYS;
-}
-
-int cfs_stub_job_schedule(sqlite3 * db)
-{
-	return ENOSYS;
-}
-
-/* vim: set noexpandtab tabstop=4: */
+/* vim: set noexpandtab tabstop=8: */
