@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2008- The University of Notre Dame
+Copyright (C) 2022 The University of Notre Dame
 This software is distributed under the GNU General Public License.
 See the file COPYING for details.
 */
@@ -18,7 +18,7 @@ See the file COPYING for details.
 #include <sys/stat.h>
 #include <dirent.h>
 
-#ifdef CCTOOLS_OPSYS_DARWIN
+#if defined (CCTOOLS_OPSYS_DARWIN) || defined(CCTOOLS_OPSYS_FREEBSD)
 #define fopen64 fopen
 #define open64 open
 #define lseek64 lseek
@@ -311,4 +311,4 @@ INT64_T chirp_recursive_put(const char *hostport, const char *source_file, const
 	return result;
 }
 
-/* vim: set noexpandtab tabstop=4: */
+/* vim: set noexpandtab tabstop=8: */
