@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2003-2004 Douglas Thain and the University of Wisconsin
-Copyright (C) 2005- The University of Notre Dame
+Copyright (C) 2022 The University of Notre Dame
 This software is distributed under the GNU General Public License.
 See the file COPYING for details.
 */
@@ -25,7 +25,7 @@ See the file COPYING for details.
 #define ABS(x) ( ((x)>=0) ? (x) : (-(x)) )
 #endif
 
-#define DIV_INT_ROUND_UP(a, b) (((a) + (b) - 1) / (b))
+#define DIV_INT_ROUND_UP(a, b) ((__typeof__(a)) ((int64_t) (((((double) (a)) + ((double) (b)) - 1) / (b)))))
 
 #define KILO 1024
 #define MEGA (KILO*KILO)
