@@ -1,34 +1,33 @@
 /*
-Copyright (C) 2013- The University of Notre Dame
+Copyright (C) 2022 The University of Notre Dame
 This software is distributed under the GNU General Public License.
 See the file COPYING for details.
 */
 
 #include <assert.h>
-#include <stdio.h>
-#include <string.h>
-#include <strings.h>
 #include <errno.h>
 #include <limits.h>
 #include <math.h>
+#include <stdio.h>
+#include <string.h>
+#include <strings.h>
 #include <sys/time.h>
 
 #include "debug.h"
-#include "path_disk_size_info.h"
+#include "load_average.h"
 #include "macros.h"
+#include "path_disk_size_info.h"
 #include "stringtools.h"
 #include "xxmalloc.h"
-#include "load_average.h"
 
-
-#include "rmonitor_poll_internal.h"
 #include "host_memory_info.h"
-#include "path_disk_size_info.h"
 #include "load_average.h"
+#include "path_disk_size_info.h"
+#include "rmonitor_poll_internal.h"
 
 /***
  * Helper functions
-***/
+ ***/
 
 #define ANON_MAPS_NAME "[anon]"
 
@@ -39,7 +38,7 @@ uint64_t usecs_since_epoch()
 
 	gettimeofday(&time, NULL);
 
-	usecs  = time.tv_sec;
+	usecs = time.tv_sec;
 	usecs *= ONE_SECOND;
 	usecs += time.tv_usec;
 
@@ -50,22 +49,21 @@ uint64_t usecs_since_epoch()
  * Functions to track the whole process tree.  They call the
  * functions defined just above, accumulating the resources of
  * all the processes.
-***/
+ ***/
 
 void rmonitor_poll_all_processes_once(struct itable *processes, struct rmonitor_process_info *acc)
 {
 	uint64_t pid;
 	struct rmonitor_process_info *p;
 
-	bzero(acc, sizeof( struct rmonitor_process_info ));
+	bzero(acc, sizeof(struct rmonitor_process_info));
 
 	itable_firstkey(processes);
-	while(itable_nextkey(processes, &pid, (void **) &p))
-	{
+	while (itable_nextkey(processes, &pid, (void **)&p)) {
 		int status = rmonitor_poll_process_once(p);
 
 		/* do not consider process if some error is found. */
-		if(status != 0)
+		if (status != 0)
 			continue;
 
 		acc_mem_usage(&acc->mem, &p->mem);
@@ -84,20 +82,19 @@ void rmonitor_poll_all_wds_once(struct hash_table *wdirs, struct rmonitor_wdir_i
 	struct rmonitor_wdir_info *d;
 	char *path;
 
-	bzero(acc, sizeof( struct rmonitor_wdir_info ));
+	bzero(acc, sizeof(struct rmonitor_wdir_info));
 
-	if(hash_table_size(wdirs) > 0) {
-		if(max_time_for_measurement > 0) {
+	if (hash_table_size(wdirs) > 0) {
+		if (max_time_for_measurement > 0) {
 			/* split time available across all directories to measure. */
-			max_time_for_measurement = MAX(1, max_time_for_measurement/hash_table_size(wdirs));
+			max_time_for_measurement = MAX(1, max_time_for_measurement / hash_table_size(wdirs));
 		}
 
 		hash_table_firstkey(wdirs);
-		while(hash_table_nextkey(wdirs, &path, (void **) &d))
-		{
+		while (hash_table_nextkey(wdirs, &path, (void **)&d)) {
 			int status = rmonitor_poll_wd_once(d, max_time_for_measurement);
 			/* do not consider directory if some error is found. */
-			if(status != 0)
+			if (status != 0)
 				continue;
 
 			acc_wd_usage(acc, d);
@@ -110,25 +107,23 @@ void rmonitor_poll_all_fss_once(struct itable *filesysms, struct rmonitor_filesy
 	struct rmonitor_filesys_info *f;
 	uint64_t dev_id;
 
-	bzero(acc, sizeof( struct rmonitor_filesys_info ));
+	bzero(acc, sizeof(struct rmonitor_filesys_info));
 
 	itable_firstkey(filesysms);
-	while(itable_nextkey(filesysms, &dev_id, (void **) &f))
-	{
+	while (itable_nextkey(filesysms, &dev_id, (void **)&f)) {
 		int status = rmonitor_poll_fs_once(f);
 		/* do not consider fs if some error is found. */
-		if(status != 0)
+		if (status != 0)
 			continue;
 
 		acc_dsk_usage(&acc->disk, &f->disk);
 	}
 }
 
-
 /***
  * Functions to monitor a single process, workind directory, or
  * filesystem.
-***/
+ ***/
 
 int rmonitor_poll_process_once(struct rmonitor_process_info *p)
 {
@@ -157,10 +152,10 @@ int rmonitor_poll_fs_once(struct rmonitor_filesys_info *f)
 {
 	int status = rmonitor_get_dsk_usage(f->path, &f->disk);
 
-	if(!status) {
-		f->disk.f_bfree  = f->disk_initial.f_bfree  - f->disk.f_bfree;
+	if (!status) {
+		f->disk.f_bfree = f->disk_initial.f_bfree - f->disk.f_bfree;
 		f->disk.f_bavail = f->disk_initial.f_bavail - f->disk.f_bavail;
-		f->disk.f_ffree  = f->disk_initial.f_ffree  - f->disk.f_ffree;
+		f->disk.f_ffree = f->disk_initial.f_ffree - f->disk.f_ffree;
 	}
 
 	return status;
@@ -172,29 +167,25 @@ int rmonitor_poll_fs_once(struct rmonitor_filesys_info *f)
 
 FILE *open_proc_file(pid_t pid, char *filename)
 {
-		FILE *fproc;
-		char fproc_path[PATH_MAX];
+	FILE *fproc;
+	char fproc_path[PATH_MAX];
 
-#if defined(CCTOOLS_OPSYS_DARWIN)
-		return NULL;
+#if defined(CCTOOLS_OPSYS_DARWIN) || defined(CCTOOLS_OPSYS_FREEBSD)
+	return NULL;
 #endif
 
-		if(pid > -1)
-		{
-			sprintf(fproc_path, "/proc/%d/%s", pid, filename);
-		}
-		else
-		{
-			sprintf(fproc_path, "/proc/%s", filename);
-		}
+	if (pid > -1) {
+		sprintf(fproc_path, "/proc/%d/%s", pid, filename);
+	} else {
+		sprintf(fproc_path, "/proc/%s", filename);
+	}
 
-		if((fproc = fopen(fproc_path, "r")) == NULL)
-		{
-				debug(D_RMON, "could not process file %s : %s\n", fproc_path, strerror(errno));
-				return NULL;
-		}
+	if ((fproc = fopen(fproc_path, "r")) == NULL) {
+		debug(D_RMON, "could not process file %s : %s\n", fproc_path, strerror(errno));
+		return NULL;
+	}
 
-		return fproc;
+	return fproc;
 }
 
 /* Parse a /proc file looking for line attribute: value */
@@ -204,23 +195,21 @@ int rmonitor_get_int_attribute(FILE *fstatus, char *attribute, uint64_t *value,
 	int not_found = 1;
 	int n = strlen(attribute);
 
-	if(!fstatus)
+	if (!fstatus)
 		return not_found;
 
 	proc_attr_line[PATH_MAX - 2] = '\0';
 	proc_attr_line[PATH_MAX - 1] = '\0';
 
-	if(rewind_flag)
+	if (rewind_flag)
 		rewind(fstatus);
 
-	while( fgets(proc_attr_line, PATH_MAX - 2, fstatus) )
-	{
-		if(strncmp(attribute, proc_attr_line, n) == 0)
-		{
-			//We make sure that fgets got a whole line
-			if(proc_attr_line[PATH_MAX - 2] == '\n')
+	while (fgets(proc_attr_line, PATH_MAX - 2, fstatus)) {
+		if (strncmp(attribute, proc_attr_line, n) == 0) {
+			// We make sure that fgets got a whole line
+			if (proc_attr_line[PATH_MAX - 2] == '\n')
 				proc_attr_line[PATH_MAX - 2] = '\0';
-			if(strlen(proc_attr_line) == PATH_MAX - 2)
+			if (strlen(proc_attr_line) == PATH_MAX - 2)
 				return -1;
 
 			sscanf(proc_attr_line, "%*s %" SCNu64, value);
@@ -253,21 +242,21 @@ int rmonitor_get_children(pid_t pid, uint64_t **children)
 	FILE *fstat = fopen(fchildren_path, "r");
 	free(fchildren_path);
 
-	if(!fstat) {
+	if (!fstat) {
 		return 0;
 	}
 
 	int count = 0;
-	int max   = 0;
+	int max = 0;
 	uint64_t child;
 
 	uint64_t *child_list = NULL;
 
-	while(fscanf(fstat, "%" PRIu64, &child) == 1) {
+	while (fscanf(fstat, "%" PRIu64, &child) == 1) {
 		count++;
-		if(count >= max) {
-			max = 2*count;
-			child_list = realloc(child_list, max*sizeof(uint64_t));
+		if (count >= max) {
+			max = 2 * count;
+			child_list = realloc(child_list, max * sizeof(uint64_t));
 		}
 
 		child_list[count - 1] = child;
@@ -280,7 +269,6 @@ int rmonitor_get_children(pid_t pid, uint64_t **children)
 	return count;
 }
 
-
 int rmonitor_get_start_time(pid_t pid, uint64_t *start_time)
 {
 	/* /proc/[pid]/stat */
@@ -289,40 +277,40 @@ int rmonitor_get_start_time(pid_t pid, uint64_t *start_time)
 	double uptime;
 
 	FILE *fstat = open_proc_file(pid, "stat");
-	if(!fstat)
+	if (!fstat)
 		return 1;
 
 	int n;
 	n = fscanf(fstat,
-			"%*s" /* pid */ "%*s" /* cmd line */ "%*s" /* state */ "%*s" /* pid of parent */
+			"%*s" /* pid */ "%*s" /* cmd line */ "%*s" /* state */ "%*s"	      /* pid of parent */
 			"%*s" /* group ID */ "%*s" /* session id */ "%*s" /* tty pid */ "%*s" /* tty group ID */
-			"%*s" /* linux/sched.h flags */ "%*s %*s %*s %*s" /* faults */
-			"%*s" /* user mode time (in clock ticks)   (field 14)  */
-			"%*s" /* kernel mode time (in clock ticks) (field 15) */
-			"%*s" /* time (clock ticks) waiting for children in user mode */
-			"%*s" /* time (clock ticks) waiting for children in kernel mode */
-			"%*s" /* priority */ "%*s" /* nice */
-			"%*s" /* num threads */ "%*s" /* always 0 */
-			"%" SCNu64 /* clock ticks since start     (field 22) */
+			"%*s" /* linux/sched.h flags */ "%*s %*s %*s %*s"		      /* faults */
+			"%*s"								      /* user mode time (in clock ticks)   (field 14)  */
+			"%*s"								      /* kernel mode time (in clock ticks) (field 15) */
+			"%*s"								      /* time (clock ticks) waiting for children in user mode */
+			"%*s"								      /* time (clock ticks) waiting for children in kernel mode */
+			"%*s" /* priority */ "%*s"					      /* nice */
+			"%*s" /* num threads */ "%*s"					      /* always 0 */
+			"%" SCNu64							      /* clock ticks since start     (field 22) */
 			/* .... */,
 			&start_clicks);
 	fclose(fstat);
 
-	if(n != 1)
+	if (n != 1)
 		return 1;
 
 	FILE *fuptime = open_proc_file(-1, "uptime");
-	if(!fuptime)
+	if (!fuptime)
 		return 1;
 
 	n = fscanf(fuptime, "%lf %*s", &uptime);
 	fclose(fuptime);
 
-	if(n != 1)
+	if (n != 1)
 		return 1;
 
 	uint64_t origin = usecs_since_epoch() - (uptime * ONE_SECOND);
-	*start_time     = origin + clicks_to_usecs(start_clicks);
+	*start_time = origin + clicks_to_usecs(start_clicks);
 
 	return 0;
 }
@@ -333,28 +321,31 @@ int rmonitor_get_cpu_time_usage(pid_t pid, struct rmonitor_cpu_time_info *cpu)
 
 	uint64_t kernel, user;
 
-
 	FILE *fstat = open_proc_file(pid, "stat");
-	if(!fstat)
+	if (!fstat)
 		return 1;
 
 	int n;
 	n = fscanf(fstat,
-			"%*s" /* pid */ "%*s" /* cmd line */ "%*s" /* state */ "%*s" /* pid of parent */
+			"%*s" /* pid */ "%*s" /* cmd line */ "%*s" /* state */ "%*s"	      /* pid of parent */
 			"%*s" /* group ID */ "%*s" /* session id */ "%*s" /* tty pid */ "%*s" /* tty group ID */
-			"%*s" /* linux/sched.h flags */ "%*s %*s %*s %*s" /* faults */
-			"%" SCNu64 /* user mode time (in clock ticks) */
-			"%" SCNu64 /* kernel mode time (in clock ticks) */
+			"%*s" /* linux/sched.h flags */ "%*s %*s %*s %*s"		      /* faults */
+			"%" SCNu64							      /* user mode time (in clock ticks) */
+			"%" SCNu64							      /* kernel mode time (in clock ticks) */
 			/* .... */,
-			&kernel, &user);
+			&kernel,
+			&user);
 	fclose(fstat);
 
-	if(n != 2)
+	if (n != 2)
 		return 1;
 
 	uint64_t accum = clicks_to_usecs(kernel) + clicks_to_usecs(user);
 
-	cpu->delta       = accum  - cpu->accumulated;
+	cpu->delta = 0;
+	if (cpu->accumulated < accum) {
+		cpu->delta = accum - cpu->accumulated;
+	}
 	cpu->accumulated = accum;
 
 	return 0;
@@ -374,7 +365,7 @@ int rmonitor_get_ctxsw_usage(pid_t pid, struct rmonitor_ctxsw_info *switches)
 	uint64_t nonvol_switches = 0;
 
 	FILE *fstat = open_proc_file(pid, "status");
-	if(!fstat) {
+	if (!fstat) {
 		return 0;
 	}
 
@@ -385,7 +376,7 @@ int rmonitor_get_ctxsw_usage(pid_t pid, struct rmonitor_ctxsw_info *switches)
 
 	uint64_t accum = vol_switches + nonvol_switches;
 
-	switches->delta       = accum  - switches->accumulated;
+	switches->delta = accum - switches->accumulated;
 	switches->accumulated = accum;
 
 	fclose(fstat);
@@ -402,15 +393,12 @@ int rmonitor_get_loadavg(struct rmonitor_load_info *load)
 	double last_minute;
 	int status = getloadavg(&last_minute, 1);
 
-	if(status != 1) {
+	if (status != 1) {
 		last_minute = -1;
 	}
 
-	int64_t tmp_output;
-	rmsummary_to_internal_unit("machine_load", last_minute, &tmp_output, "procs");
-
-	load->last_minute = tmp_output;
-	load->cpus        = load_average_get_cpus();
+	load->last_minute = last_minute;
+	load->cpus = load_average_get_cpus();
 
 	return 0;
 }
@@ -420,16 +408,16 @@ int rmonitor_get_mem_usage(pid_t pid, struct rmonitor_mem_info *mem)
 	// /proc/[pid]/status:
 
 	FILE *fmem = open_proc_file(pid, "status");
-	if(!fmem)
+	if (!fmem)
 		return 1;
 
 	int status = 0;
 	/* in kB */
-	status |= rmonitor_get_int_attribute(fmem, "VmPeak:", &mem->virtual,  1);
-	status |= rmonitor_get_int_attribute(fmem, "VmHWM:",  &mem->resident, 1);
-	status |= rmonitor_get_int_attribute(fmem, "VmLib:",  &mem->shared,   1);
-	status |= rmonitor_get_int_attribute(fmem, "VmExe:",  &mem->text,     1);
-	status |= rmonitor_get_int_attribute(fmem, "VmData:", &mem->data,     1);
+	status |= rmonitor_get_int_attribute(fmem, "VmPeak:", &mem->virtual, 1);
+	status |= rmonitor_get_int_attribute(fmem, "VmHWM:", &mem->resident, 1);
+	status |= rmonitor_get_int_attribute(fmem, "VmLib:", &mem->shared, 1);
+	status |= rmonitor_get_int_attribute(fmem, "VmExe:", &mem->text, 1);
+	status |= rmonitor_get_int_attribute(fmem, "VmData:", &mem->data, 1);
 
 	/* from smaps when reading maps. */
 	mem->swap = 0;
@@ -437,31 +425,32 @@ int rmonitor_get_mem_usage(pid_t pid, struct rmonitor_mem_info *mem)
 	fclose(fmem);
 
 	/* in MB */
-	mem->virtual  = DIV_INT_ROUND_UP(mem->virtual,  1024);
+	mem->virtual = DIV_INT_ROUND_UP(mem->virtual, 1024);
 	mem->resident = DIV_INT_ROUND_UP(mem->resident, 1024);
-	mem->text     = DIV_INT_ROUND_UP(mem->text,     1024);
-	mem->data     = DIV_INT_ROUND_UP(mem->data,     1024);
-	mem->shared   = DIV_INT_ROUND_UP(mem->shared,   1024);
+	mem->text = DIV_INT_ROUND_UP(mem->text, 1024);
+	mem->data = DIV_INT_ROUND_UP(mem->data, 1024);
+	mem->shared = DIV_INT_ROUND_UP(mem->shared, 1024);
 
 	return status;
 }
 
 void acc_mem_usage(struct rmonitor_mem_info *acc, struct rmonitor_mem_info *other)
 {
-		acc->virtual  += other->virtual;
-		acc->resident += other->resident;
-		acc->data     += other->data;
-		acc->swap     += other->swap;
-		acc->shared   += other->shared;
+	acc->virtual += other->virtual;
+	acc->resident += other->resident;
+	acc->data += other->data;
+	acc->swap += other->swap;
+	acc->shared += other->shared;
 }
 
-struct rmonitor_mem_info *rmonitor_get_map_info(FILE *fmem, int rewind_flag) {
+struct rmonitor_mem_info *rmonitor_get_map_info(FILE *fmem, int rewind_flag)
+{
 	static int anon_map_count = 0;
 
-	if(!fmem)
+	if (!fmem)
 		return NULL;
 
-	if(rewind_flag)
+	if (rewind_flag)
 		rewind(fmem);
 
 	struct rmonitor_mem_info *info = malloc(sizeof(struct rmonitor_mem_info));
@@ -469,20 +458,24 @@ struct rmonitor_mem_info *rmonitor_get_map_info(FILE *fmem, int rewind_flag) {
 	uint64_t offset;
 	char map_info_line[PATH_MAX];
 	char map_name_found[PATH_MAX];
-	while( fgets(map_info_line, PATH_MAX, fmem) )
-	{
+	while (fgets(map_info_line, PATH_MAX, fmem)) {
 		// start-end                 perm   offset device inode                       path
 		// 560019f25000-56001a127000 r-xp 00000000 08:01 266469                     /usr/bin/vim.basic
 
 		int n;
-		n = sscanf(map_info_line, "%llx-%llx %*s %llx %*s %*s %s", (long long unsigned int *) &(info->map_start), (long long unsigned int *)  &(info->map_end), (long long unsigned int *) &offset, map_name_found);
+		n = sscanf(map_info_line,
+				"%llx-%llx %*s %llx %*s %*s %s",
+				(long long unsigned int *)&(info->map_start),
+				(long long unsigned int *)&(info->map_end),
+				(long long unsigned int *)&offset,
+				map_name_found);
 
 		/* continue if we do not get at least start, end, and offset */
-		if(n < 3)
+		if (n < 3)
 			continue;
 
 		/* file maps are always an absolute pathname. consider maps without a filename as different. */
-		if(n < 4 || map_name_found[0] != '/') {
+		if (n < 4 || map_name_found[0] != '/') {
 			info->map_name = string_format("ANON_MAPS_NAME.%d", anon_map_count);
 			anon_map_count++;
 		} else {
@@ -490,7 +483,7 @@ struct rmonitor_mem_info *rmonitor_get_map_info(FILE *fmem, int rewind_flag) {
 		}
 
 		// move boundaries to origin
-		info->map_end   = info->map_end - info->map_start + offset;
+		info->map_end = info->map_end - info->map_start + offset;
 		info->map_start = offset;
 
 		return info;
@@ -501,10 +494,11 @@ struct rmonitor_mem_info *rmonitor_get_map_info(FILE *fmem, int rewind_flag) {
 	return NULL;
 }
 
-static double rmonitor_mem_info_priority(void *item) {
+static double rmonitor_mem_info_priority(void *item)
+{
 	assert(item);
 	struct rmonitor_mem_info *i = item;
-	return -1*(i->map_start);
+	return -1 * (i->map_start);
 }
 
 int rmonitor_get_mmaps_usage(pid_t pid, struct hash_table *maps)
@@ -512,11 +506,11 @@ int rmonitor_get_mmaps_usage(pid_t pid, struct hash_table *maps)
 	// /proc/[pid]/smaps:
 
 	FILE *fmem = open_proc_file(pid, "smaps");
-	if(!fmem)
+	if (!fmem)
 		return 1;
 
 	struct rmonitor_mem_info *info;
-	while((info = rmonitor_get_map_info(fmem, 0))) {
+	while ((info = rmonitor_get_map_info(fmem, 0))) {
 
 		uint64_t rss, pss, swap, ref;
 		uint64_t private_dirty, private_clean;
@@ -525,22 +519,22 @@ int rmonitor_get_mmaps_usage(pid_t pid, struct hash_table *maps)
 
 		/* order is important, this is how the fields appear in smaps */
 		/* in kB! */
-		status |= rmonitor_get_int_attribute(fmem, "Rss:",           &rss, 0);
-		status |= rmonitor_get_int_attribute(fmem, "Pss:",           &pss, 0);
+		status |= rmonitor_get_int_attribute(fmem, "Rss:", &rss, 0);
+		status |= rmonitor_get_int_attribute(fmem, "Pss:", &pss, 0);
 		status |= rmonitor_get_int_attribute(fmem, "Private_Clean:", &private_clean, 0);
 		status |= rmonitor_get_int_attribute(fmem, "Private_Dirty:", &private_dirty, 0);
-		status |= rmonitor_get_int_attribute(fmem, "Referenced:",    &ref, 0);
-		status |= rmonitor_get_int_attribute(fmem, "Swap:",          &swap, 0);
+		status |= rmonitor_get_int_attribute(fmem, "Referenced:", &ref, 0);
+		status |= rmonitor_get_int_attribute(fmem, "Swap:", &swap, 0);
 
 		/* error reading a field, we simply skip the record. */
-		if(status) {
+		if (status) {
 			free(info);
 			continue;
 		}
 
-		info->resident   = rss;
+		info->resident = rss;
 		info->referenced = ref;
-		info->swap       = swap;
+		info->swap = swap;
 
 		/* private and shared may or may not be currently resident, (e.g.,
 		 swap). That is: rss = private + shared - swap = referenced - swap.  In
@@ -549,12 +543,12 @@ int rmonitor_get_mmaps_usage(pid_t pid, struct hash_table *maps)
 		 case that all private pages are resident. If swap is zero, then
 		 resident private and resident shared will have the correct values. */
 
-		info->private  = MIN(private_dirty + private_clean, rss);
-		info->shared   = MAX(rss - info->private, 0);
+		info->private = MIN(private_dirty + private_clean, rss);
+		info->shared = MAX(rss - info->private, 0);
 
 		/* add the info to a sorted list per map, by start. Overlaping maps will be merged later. */
 		struct list *infos = hash_table_lookup(maps, info->map_name);
-		if(!infos) {
+		if (!infos) {
 			infos = list_create();
 			hash_table_insert(maps, info->map_name, infos);
 		}
@@ -567,7 +561,8 @@ int rmonitor_get_mmaps_usage(pid_t pid, struct hash_table *maps)
 	return 0;
 }
 
-int rmonitor_poll_maps_once(struct itable *processes, struct rmonitor_mem_info *mem) {
+int rmonitor_poll_maps_once(struct itable *processes, struct rmonitor_mem_info *mem)
+{
 	/* set result to 0. */
 	bzero(mem, sizeof(struct rmonitor_mem_info));
 
@@ -576,7 +571,7 @@ int rmonitor_poll_maps_once(struct itable *processes, struct rmonitor_mem_info *
 	uint64_t pid;
 	struct rmonitor_process_info *pinfo;
 	itable_firstkey(processes);
-	while(itable_nextkey(processes, &pid, (void *) &pinfo)) {
+	while (itable_nextkey(processes, &pid, (void *)&pinfo)) {
 		rmonitor_get_mmaps_usage(pid, maps_per_file);
 	}
 
@@ -604,23 +599,23 @@ int rmonitor_poll_maps_once(struct itable *processes, struct rmonitor_mem_info *
 	struct list *infos;
 
 	hash_table_firstkey(maps_per_file);
-	while(hash_table_nextkey(maps_per_file, &map_name, (void *) &infos )) {
+	while (hash_table_nextkey(maps_per_file, &map_name, (void *)&infos)) {
 
 		struct rmonitor_mem_info *info, *next;
-		while((info = list_pop_head(infos))) {
-			while((next = list_peek_head(infos))) {
+		while ((info = list_pop_head(infos))) {
+			while ((next = list_peek_head(infos))) {
 				/* do we need to merge with the next segment? */
-				if(info->map_end > next->map_start) {
-					info->private  += next->private;
-					info->shared   += next->shared;
+				if (info->map_end > next->map_start) {
+					info->private += next->private;
+					info->shared += next->shared;
 					info->resident += next->resident;
 					info->referenced += next->referenced;
-					info->swap     += next->swap;
+					info->swap += next->swap;
 
 					info->map_end = MAX(info->map_end, next->map_end);
 
 					list_pop_head(infos);
-					if(next->map_name)
+					if (next->map_name)
 						free(next->map_name);
 					free(next);
 				} else {
@@ -631,30 +626,30 @@ int rmonitor_poll_maps_once(struct itable *processes, struct rmonitor_mem_info *
 			/* a series of upper bounds: */
 			/* by adding referenced, we assumed a worst case of non-sharing
 			 * memory, but referenced cannot be larger than the virtual size: */
-			info->virtual  = DIV_INT_ROUND_UP(info->map_end - info->map_start, 1024); /* bytes to kB. */
+			info->virtual = DIV_INT_ROUND_UP(info->map_end - info->map_start, 1024); /* bytes to kB. */
 			info->referenced = MIN(info->referenced, info->virtual);
 
 			/* similarly, resident cannot be larger than referenced. */
 			info->resident = MIN(info->resident, info->referenced);
 
 			/* and, resident private cannot be larger than resident. */
-			info->private  = MIN(info->private, info->resident);
+			info->private = MIN(info->private, info->resident);
 
 			/* lastly, resident shared memory cannot be larger than the whole
 			 * resident size minus the resident private memory. */
 			info->shared = MIN(info->shared, info->resident - info->private);
 
 			/* once the individual values have been found, we added together to the result. */
-			mem->virtual     += info->virtual;
-			mem->referenced  += info->referenced;
-			mem->shared      += info->shared;
-			mem->private     += info->private;
+			mem->virtual += info->virtual;
+			mem->referenced += info->referenced;
+			mem->shared += info->shared;
+			mem->private += info->private;
 
 			/* note that we add private + shared, rather than resident,
 			 * otherwise we will overcount shared. */
 			mem->resident += info->private + info->shared;
 
-			if(info->map_name)
+			if (info->map_name)
 				free(info->map_name);
 			free(info);
 		}
@@ -665,15 +660,14 @@ int rmonitor_poll_maps_once(struct itable *processes, struct rmonitor_mem_info *
 	hash_table_delete(maps_per_file);
 
 	/* all the values computed are in kB, we convert to MB. */
-	mem->virtual      = DIV_INT_ROUND_UP(mem->virtual,  1024);
-	mem->shared       = DIV_INT_ROUND_UP(mem->shared,   1024);
-	mem->private      = DIV_INT_ROUND_UP(mem->private,  1024);
-	mem->resident     = DIV_INT_ROUND_UP(mem->resident, 1024);
+	mem->virtual = DIV_INT_ROUND_UP(mem->virtual, 1024);
+	mem->shared = DIV_INT_ROUND_UP(mem->shared, 1024);
+	mem->private = DIV_INT_ROUND_UP(mem->private, 1024);
+	mem->resident = DIV_INT_ROUND_UP(mem->resident, 1024);
 
 	return 0;
 }
 
-
 int rmonitor_get_sys_io_usage(pid_t pid, struct rmonitor_io_info *io)
 {
 	/* /proc/[pid]/io: if process dies before we read the file,
@@ -688,21 +682,21 @@ int rmonitor_get_sys_io_usage(pid_t pid, struct rmonitor_io_info *io)
 	io->delta_chars_read = 0;
 	io->delta_chars_written = 0;
 
-	if(!fio)
+	if (!fio)
 		return 1;
 
 	/* We really want "bytes_read", but there are issues with
 	 * distributed filesystems. Instead, we also count page
 	 * faulting in another function below. */
-	rstatus  = rmonitor_get_int_attribute(fio, "rchar", &cread, 1);
-	wstatus  = rmonitor_get_int_attribute(fio, "write_bytes", &cwritten, 1);
+	rstatus = rmonitor_get_int_attribute(fio, "rchar", &cread, 1);
+	wstatus = rmonitor_get_int_attribute(fio, "write_bytes", &cwritten, 1);
 
 	fclose(fio);
 
-	if(rstatus || wstatus)
+	if (rstatus || wstatus)
 		return 1;
 
-	io->delta_chars_read    = cread    - io->chars_read;
+	io->delta_chars_read = cread - io->chars_read;
 	io->delta_chars_written = cwritten - io->chars_written;
 
 	io->chars_read = cread;
@@ -713,7 +707,7 @@ int rmonitor_get_sys_io_usage(pid_t pid, struct rmonitor_io_info *io)
 
 void acc_sys_io_usage(struct rmonitor_io_info *acc, struct rmonitor_io_info *other)
 {
-	acc->delta_chars_read    += other->delta_chars_read;
+	acc->delta_chars_read += other->delta_chars_read;
 	acc->delta_chars_written += other->delta_chars_written;
 }
 
@@ -725,24 +719,23 @@ int rmonitor_get_map_io_usage(pid_t pid, struct rmonitor_io_info *io)
 	uint64_t kbytes_resident_accum;
 	uint64_t kbytes_resident;
 
-	kbytes_resident_accum    = 0;
+	kbytes_resident_accum = 0;
 	io->delta_bytes_faulted = 0;
 
 	FILE *fsmaps = open_proc_file(pid, "smaps");
-	if(!fsmaps)
-	{
+	if (!fsmaps) {
 		return 1;
 	}
 
 	char dummy_line[1024];
 
 	/* Look for next mmap file */
-	while(fgets(dummy_line, 1024, fsmaps))
-		if(strchr(dummy_line, '/'))
-			if(rmonitor_get_int_attribute(fsmaps, "Rss:", &kbytes_resident, 0) == 0)
+	while (fgets(dummy_line, 1024, fsmaps))
+		if (strchr(dummy_line, '/'))
+			if (rmonitor_get_int_attribute(fsmaps, "Rss:", &kbytes_resident, 0) == 0)
 				kbytes_resident_accum += kbytes_resident;
 
-	if((kbytes_resident_accum * 1024) > io->bytes_faulted)
+	if ((kbytes_resident_accum * 1024) > io->bytes_faulted)
 		io->delta_bytes_faulted = (kbytes_resident_accum * 1024) - io->bytes_faulted;
 
 	/* in bytes */
@@ -758,7 +751,6 @@ void acc_map_io_usage(struct rmonitor_io_info *acc, struct rmonitor_io_info *oth
 	acc->delta_bytes_faulted += other->delta_bytes_faulted;
 }
 
-
 /***
  * Low level resource monitor functions.
  ***/
@@ -769,8 +761,7 @@ int rmonitor_get_dsk_usage(const char *path, struct statfs *disk)
 
 	debug(D_RMON, "statfs on path: %s\n", path);
 
-	if(statfs(path, disk) > 0)
-	{
+	if (statfs(path, disk) > 0) {
 		debug(D_RMON, "could not statfs on %s : %s\n", cwd, strerror(errno));
 		return 1;
 	}
@@ -780,9 +771,9 @@ int rmonitor_get_dsk_usage(const char *path, struct statfs *disk)
 
 void acc_dsk_usage(struct statfs *acc, struct statfs *other)
 {
-	acc->f_bfree  += other->f_bfree;
+	acc->f_bfree += other->f_bfree;
 	acc->f_bavail += other->f_bavail;
-	acc->f_ffree  += other->f_ffree;
+	acc->f_ffree += other->f_ffree;
 }
 
 int rmonitor_get_wd_usage(struct rmonitor_wdir_info *d, int max_time_for_measurement)
@@ -801,8 +792,8 @@ int rmonitor_get_wd_usage(struct rmonitor_wdir_info *d, int max_time_for_measure
 
 void acc_wd_usage(struct rmonitor_wdir_info *acc, struct rmonitor_wdir_info *other)
 {
-	acc->files       += other->files;
-	acc->byte_count  += other->byte_count;
+	acc->files += other->files;
+	acc->byte_count += other->byte_count;
 }
 
 char *rmonitor_get_command_line(pid_t pid)
@@ -810,110 +801,118 @@ char *rmonitor_get_command_line(pid_t pid)
 	/* /proc/[pid]/cmdline */
 
 	FILE *fline = open_proc_file(pid, "cmdline");
-	if(!fline)
+	if (!fline)
 		return NULL;
 
 	char cmdline[PATH_MAX];
 	ssize_t cmdline_len = read(fileno(fline), cmdline, PATH_MAX);
 
-	if(cmdline_len < 1)
+	if (cmdline_len < 1)
 		return NULL;
 
 	int i;
-	for(i=0; i < cmdline_len - 1; i++) { /* -1 because cmdline ends with two \0. */
-		if(cmdline[i] == '\0')
+	for (i = 0; i < cmdline_len - 1; i++) { /* -1 because cmdline ends with two \0. */
+		if (cmdline[i] == '\0')
 			cmdline[i] = ' ';
 	}
-        fclose(fline);
+	fclose(fline);
 
 	return xxstrdup(cmdline);
 }
 
 void rmonitor_info_to_rmsummary(struct rmsummary *tr, struct rmonitor_process_info *p, struct rmonitor_wdir_info *d, struct rmonitor_filesys_info *f, uint64_t start_time)
 {
-	tr->start        = start_time;
-	tr->end          = usecs_since_epoch();
-	tr->wall_time    = tr->end - tr->start;
-	tr->cpu_time     = p->cpu.accumulated;
-	tr->cores        = 0;
-	tr->cores_avg    = 0;
+	tr->start = ((double)start_time) / ONE_SECOND;
+	tr->end = ((double)usecs_since_epoch()) / ONE_SECOND;
 
-	if(tr->wall_time > 0) {
+	tr->wall_time = tr->end - tr->start;
+	tr->cpu_time = ((double)p->cpu.accumulated) / ONE_SECOND;
 
-		int64_t tmp_output;
-		rmsummary_to_internal_unit("cores", ((double) tr->cpu_time)/tr->wall_time, &tmp_output, "cores");
-		tr->cores = tmp_output;
+	tr->cores = 0;
+	tr->cores_avg = 0;
 
-		rmsummary_to_internal_unit("cores_avg", ((double) tr->cpu_time)/tr->wall_time, &tmp_output, "cores");
-		tr->cores_avg = tmp_output;
+	/* set both cores and cores_avg to avg, as info does not come from time windows. */
+	if (tr->wall_time > 0 && tr->cpu_time >= 0) {
+		// set cores = cpu_time/wall_time;
+		tr->cores = DIV_INT_ROUND_UP(tr->cpu_time, tr->wall_time);
+		tr->cores_avg = tr->cores;
 	}
 
 	tr->context_switches = p->ctx.accumulated;
-
 	tr->max_concurrent_processes = -1;
-	tr->total_processes          = -1;
+	tr->total_processes = -1;
 
-	tr->virtual_memory = (int64_t) p->mem.virtual;
-	tr->memory         = (int64_t) p->mem.resident;
-	tr->swap_memory    = (int64_t) p->mem.swap;
+	tr->virtual_memory = p->mem.virtual;
+	tr->memory = p->mem.resident;
+	tr->swap_memory = p->mem.virtual;
 
-	tr->bytes_read        = (int64_t)  p->io.chars_read;
-	tr->bytes_written     = (int64_t)  p->io.chars_written;
+	// assigning values read/written in MB
+	tr->bytes_read = ((double)(p->io.chars_read + p->io.bytes_faulted)) / ONE_MEGABYTE;
+	tr->bytes_written = ((double)p->io.chars_written) / ONE_MEGABYTE;
 
-	tr->total_files = -1;
-	tr->disk = -1;
+	tr->machine_load = p->load.last_minute;
+	tr->machine_cpus = p->load.cpus;
 
-	if(d) {
-		tr->total_files = (int64_t) (d->files);
-		tr->disk        = (int64_t) (d->byte_count + ONE_MEGABYTE - 1) / ONE_MEGABYTE;
+	if (d) {
+		tr->total_files = d->files;
+		tr->disk = ((double)d->byte_count) / ONE_MEGABYTE;
+	} else {
+		tr->total_files = -1;
+		tr->disk = -1;
 	}
 
-	tr->fs_nodes = -1;
-	if(f) {
-		tr->fs_nodes          = (int64_t) f->disk.f_ffree;
+	if (f) {
+		tr->fs_nodes = f->disk.f_ffree;
+	} else {
+		tr->fs_nodes = -1;
 	}
 
 	tr->machine_load = p->load.last_minute;
 	tr->machine_cpus = p->load.cpus;
 }
 
-struct rmsummary *rmonitor_measure_process(pid_t pid) {
+struct rmsummary *rmonitor_measure_process(pid_t pid, int include_disk)
+{
 	int err;
 
 	struct rmsummary *tr = rmsummary_create(-1);
 
 	struct rmonitor_process_info p;
+	memset(&p, 0, sizeof(p));
 	p.pid = pid;
 
 	err = rmonitor_poll_process_once(&p);
-	if(err != 0)
+	if (err != 0)
 		return NULL;
 
-	char cwd_link[PATH_MAX];
-	char cwd_org[PATH_MAX];
-
 	struct rmonitor_wdir_info *d = NULL;
-	snprintf(cwd_link, PATH_MAX, "/proc/%d/cwd", pid);
-	ssize_t n = readlink(cwd_link, cwd_org, PATH_MAX - 1);
 
-	if(n != -1)  {
-		cwd_org[n] = '\0';
-		d = malloc(sizeof(struct rmonitor_wdir_info));
-		d->path  = cwd_org;
-		d->state = NULL;
+	if (include_disk) {
+		char cwd_link[PATH_MAX];
+		char cwd_org[PATH_MAX];
 
-		rmonitor_poll_wd_once(d, -1);
+		snprintf(cwd_link, PATH_MAX, "/proc/%d/cwd", pid);
+		ssize_t n = readlink(cwd_link, cwd_org, PATH_MAX - 1);
+
+		if (n != -1) {
+			cwd_org[n] = '\0';
+			d = malloc(sizeof(struct rmonitor_wdir_info));
+			d->path = cwd_org;
+			d->state = NULL;
+
+			rmonitor_poll_wd_once(d, -1);
+		}
 	}
 
 	uint64_t start;
 	err = rmonitor_get_start_time(pid, &start);
-	if(err != 0)
+	if (err != 0)
 		return NULL;
 
 	rmonitor_info_to_rmsummary(tr, &p, d, NULL, start);
 	tr->command = rmonitor_get_command_line(pid);
 
-	if(d) {
+	if (d) {
 		path_disk_size_info_delete_state(d->state);
 		free(d);
 	}
@@ -921,12 +920,14 @@ struct rmsummary *rmonitor_measure_process(pid_t pid) {
 	return tr;
 }
 
-int rmonitor_measure_process_update_to_peak(struct rmsummary *tr, pid_t pid) {
+int rmonitor_measure_process_update_to_peak(struct rmsummary *tr, pid_t pid, int include_disk)
+{
 
-	struct rmsummary *now = rmonitor_measure_process(pid);
+	struct rmsummary *now = rmonitor_measure_process(pid, include_disk);
 
-	if(!now)
+	if (!now) {
 		return 0;
+	}
 
 	rmsummary_merge_max(tr, now);
 
@@ -935,69 +936,74 @@ int rmonitor_measure_process_update_to_peak(struct rmsummary *tr, pid_t pid) {
 	return 1;
 }
 
-struct rmsummary *rmonitor_measure_host(char *path) {
+struct rmsummary *rmonitor_measure_host(char *path)
+{
 	uint64_t free_mem;
 	uint64_t total_mem;
 	int64_t total_disk;
 	int64_t file_count;
 	struct rmsummary *tr = rmsummary_create(-1);
 
-	if(path) {
+	if (path) {
 		path_disk_size_info_get(path, &total_disk, &file_count);
-		tr->disk = total_disk / MEGABYTE;
+		tr->disk = ((double)total_disk) / ONE_MEGABYTE;
 		tr->total_files = file_count;
 	}
+
 	host_memory_info_get(&free_mem, &total_mem);
-	tr->memory = total_mem / MEGABYTE;
+	tr->memory = ((double)total_mem) / ONE_MEGABYTE;
 	tr->cores = load_average_get_cpus();
 	rmsummary_read_env_vars(tr);
 
 	return tr;
 }
 
-struct rmsummary *rmonitor_collate_minimonitor(uint64_t start_time, int current_ps, int total_processes, struct rmonitor_process_info *p, struct rmonitor_mem_info *m, struct rmonitor_wdir_info *d) {
+struct rmsummary *rmonitor_collate_minimonitor(
+		uint64_t start_time, int current_ps, int total_processes, struct rmonitor_process_info *p, struct rmonitor_mem_info *m, struct rmonitor_wdir_info *d)
+{
 	struct rmsummary *tr = rmsummary_create(-1);
 
-	tr->wall_time  = usecs_since_epoch() - start_time;
-	tr->cpu_time   = p->cpu.accumulated;
+	tr->start = ((double)start_time) / ONE_SECOND;
+	tr->end = ((double)usecs_since_epoch()) / ONE_SECOND;
 
-	tr->start = start_time;
-	tr->end   = usecs_since_epoch();
+	tr->wall_time = tr->end - tr->start;
+	tr->cpu_time = ((double)p->cpu.accumulated) / ONE_SECOND;
 
 	tr->cores = 0;
 
-	if(tr->wall_time > 0) {
+	if (tr->wall_time > 0) {
+		// set cores = cpu_time/wall_time
 		tr->cores = DIV_INT_ROUND_UP(tr->cpu_time, tr->wall_time);
 	}
 
-	tr->max_concurrent_processes = (int64_t) current_ps;
-	tr->total_processes          = (int64_t) total_processes;
+	tr->context_switches = p->ctx.accumulated;
+	tr->max_concurrent_processes = current_ps;
+	tr->total_processes = total_processes;
 
 	/* we use max here, as /proc/pid/smaps that fills *m is not always
 	 * available. This causes /proc/pid/status to become a conservative
 	 * fallback. */
-	if(m->resident > 0) {
-		tr->virtual_memory    = (int64_t) m->virtual;
-		tr->memory            = (int64_t) m->resident;
-		tr->swap_memory       = (int64_t) m->swap;
-	}
-	else {
-		tr->virtual_memory    = (int64_t) p->mem.virtual;
-		tr->memory            = (int64_t) p->mem.resident;
-		tr->swap_memory       = (int64_t) p->mem.swap;
+	if (m->resident > 0) {
+		tr->virtual_memory = m->virtual;
+		tr->memory = m->resident;
+		tr->swap_memory = m->virtual;
+	} else {
+		tr->virtual_memory = p->mem.virtual;
+		tr->memory = p->mem.resident;
+		tr->swap_memory = p->mem.virtual;
 	}
 
-	tr->bytes_read        = (int64_t) p->io.chars_read;
-	tr->bytes_read       += (int64_t) p->io.bytes_faulted;
-	tr->bytes_written     = (int64_t) p->io.chars_written;
+	// assigning values read/written in MB
+	tr->bytes_read = ((double)(p->io.chars_read + p->io.bytes_faulted)) / ONE_MEGABYTE;
+	tr->bytes_written = ((double)p->io.chars_written) / ONE_MEGABYTE;
 
 	// set in resource_monitor.c from messages of the helper
 	// tr->bytes_received = total_bytes_rx;
 	// tr->bytes_sent     = total_bytes_tx;
 	// tr->bandwidth = average_bandwidth(1);
 
-	tr->total_files = (int64_t) d->files;
-	tr->disk        = (int64_t) DIV_INT_ROUND_UP(d->byte_count, ONE_MEGABYTE);
+	tr->total_files = d->files;
+	tr->disk = ((double)d->byte_count) / ONE_MEGABYTE;
 
 	tr->machine_load = p->load.last_minute;
 	tr->machine_cpus = p->load.cpus;
@@ -1005,89 +1011,90 @@ struct rmsummary *rmonitor_collate_minimonitor(uint64_t start_time, int current_
 	return tr;
 }
 
-struct rmsummary *rmonitor_minimonitor(minimonitor_op op, uint64_t pid) {
-	static struct itable                 *processes = NULL;
-	static struct rmonitor_process_info *p_acc      = NULL;
-	static struct rmonitor_mem_info     *m_acc      = NULL;
-	static struct rmonitor_wdir_info    *d_acc      = NULL;
+struct rmsummary *rmonitor_minimonitor(minimonitor_op op, uint64_t pid)
+{
+	static struct itable *processes = NULL;
+	static struct rmonitor_process_info *p_acc = NULL;
+	static struct rmonitor_mem_info *m_acc = NULL;
+	static struct rmonitor_wdir_info *d_acc = NULL;
 
-	static uint64_t first_pid  = 0;
+	static uint64_t first_pid = 0;
 	static uint64_t start_time = 0;
 	static uint64_t total_processes = 0;
 
 	struct rmsummary *result = NULL;
 
-	if(!processes) {
+	if (!processes) {
 		processes = itable_create(0);
-		p_acc     = calloc(1, sizeof(*p_acc)); //Automatic zeroed.
-		m_acc     = calloc(1, sizeof(*m_acc));
-		d_acc     = calloc(1, sizeof(*d_acc));
+		p_acc = calloc(1, sizeof(*p_acc)); // Automatic zeroed.
+		m_acc = calloc(1, sizeof(*m_acc));
+		d_acc = calloc(1, sizeof(*d_acc));
 	}
 
 	char cwd_link[PATH_MAX];
 	char cwd_org[PATH_MAX];
 
 	struct rmonitor_process_info *p;
-	switch(op) {
-		case MINIMONITOR_RESET:
-			if(processes) {
-				itable_firstkey(processes);
-				while(itable_nextkey(processes, &pid, (void **) &p)) {
-					itable_remove(processes, pid);
-					free(p);
-				}
-				first_pid = 0;
-				total_processes = 0;
-				memset(p_acc, 0, sizeof(*p_acc));
-				memset(m_acc, 0, sizeof(*m_acc));
-				path_disk_size_info_delete_state(d_acc->state);
-			}
-			break;
-		case MINIMONITOR_ADD_PID:
-			p = itable_lookup(processes, pid);
-			if(!p) {
-				p = calloc(1, sizeof(struct rmonitor_process_info));
-				p->pid = pid;
-				itable_insert(processes, p->pid, (void *) p);
-				total_processes++;
-				if(first_pid == 0) {
-					first_pid = pid;
-					if(start_time < 1) {
-						rmonitor_get_start_time(pid, &start_time);
-					}
-
-					snprintf(cwd_link, PATH_MAX, "/proc/%" PRIu64 "/cwd", pid);
-					size_t n = readlink(cwd_link, cwd_org, PATH_MAX - 1);
-					if(n > 0)  {
-						cwd_org[n] = '\0';
-						d_acc->path  = cwd_org;
-						d_acc->state = NULL;
-					}
-				}
-			}
-			break;
-		case MINIMONITOR_REMOVE_PID:
-			p = itable_lookup(processes, pid);
-			if(p) {
+	switch (op) {
+	case MINIMONITOR_RESET:
+		if (processes) {
+			itable_firstkey(processes);
+			while (itable_nextkey(processes, &pid, (void **)&p)) {
 				itable_remove(processes, pid);
 				free(p);
-				if(pid == first_pid) {
-					first_pid = 0;
+			}
+			first_pid = 0;
+			total_processes = 0;
+			memset(p_acc, 0, sizeof(*p_acc));
+			memset(m_acc, 0, sizeof(*m_acc));
+			path_disk_size_info_delete_state(d_acc->state);
+		}
+		break;
+	case MINIMONITOR_ADD_PID:
+		p = itable_lookup(processes, pid);
+		if (!p) {
+			p = calloc(1, sizeof(struct rmonitor_process_info));
+			p->pid = pid;
+			itable_insert(processes, p->pid, (void *)p);
+			total_processes++;
+			if (first_pid == 0) {
+				first_pid = pid;
+				if (start_time < 1) {
+					rmonitor_get_start_time(pid, &start_time);
+				}
+
+				snprintf(cwd_link, PATH_MAX, "/proc/%" PRIu64 "/cwd", pid);
+				size_t n = readlink(cwd_link, cwd_org, PATH_MAX - 1);
+				if (n > 0) {
+					cwd_org[n] = '\0';
+					d_acc->path = cwd_org;
+					d_acc->state = NULL;
 				}
 			}
-			break;
-		case MINIMONITOR_MEASURE:
-			if(itable_size(processes) > 0) {
-				rmonitor_poll_all_processes_once(processes, p_acc);
-				rmonitor_poll_maps_once(processes, m_acc);
-				rmonitor_poll_wd_once(d_acc, 1);
-
-				result = rmonitor_collate_minimonitor(start_time, itable_size(processes), total_processes, p_acc, m_acc, d_acc);
+		}
+		break;
+	case MINIMONITOR_REMOVE_PID:
+		p = itable_lookup(processes, pid);
+		if (p) {
+			itable_remove(processes, pid);
+			free(p);
+			if (pid == first_pid) {
+				first_pid = 0;
 			}
-			break;
+		}
+		break;
+	case MINIMONITOR_MEASURE:
+		if (itable_size(processes) > 0) {
+			rmonitor_poll_all_processes_once(processes, p_acc);
+			rmonitor_poll_maps_once(processes, m_acc);
+			rmonitor_poll_wd_once(d_acc, 1);
+
+			result = rmonitor_collate_minimonitor(start_time, itable_size(processes), total_processes, p_acc, m_acc, d_acc);
+		}
+		break;
 	}
 
 	return result;
 }
 
-/* vim: set noexpandtab tabstop=4: */
+/* vim: set noexpandtab tabstop=8: */
