@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2013- The University of Notre Dame
+Copyright (C) 2022 The University of Notre Dame
 This software is distributed under the GNU General Public License.
 See the file COPYING for details.
 */
@@ -692,8 +692,6 @@ void dag_to_dot(struct dag *d, int condense_display, int change_size, int with_l
 	char *label;
 
 	//Dot Details Variables
-	int i;
-	int j;
 	struct file_node *src;
 	struct file_node *tar;
 
@@ -790,19 +788,18 @@ void dag_to_dot(struct dag *d, int condense_display, int change_size, int with_l
 			if(with_details) {
 				printf("subgraph cluster_S%d { \n", condense_display ? t->id : n->nodeid);
 				printf("\tstyle=unfilled;\n\tcolor=red\n");
-				printf("\tcores%d [style=filled, color=white, label=\"Cores: %"PRId64"\"]\n", condense_display ? t->id : n->nodeid, n->resources_requested->cores);
-				printf("\tresMem%d [style=filled, color=white, label=\"Memory: %"PRId64" MB\"]\n", condense_display ? t->id : n->nodeid, n->resources_requested->memory);
-				printf("\tworkDirFtprnt%d [style=filled, color=white, label=\"Footprint: %"PRId64" MB\"]\n", condense_display ? t->id : n->nodeid, n->resources_requested->disk);
+				printf("\tcores%d [style=filled, color=white, label=\"cores: %s\"]\n", condense_display ? t->id : n->nodeid, rmsummary_resource_to_str("cores", n->resources_requested->cores, 0));
+				printf("\tgpus%d [style=filled, color=white, label=\"gpus: %s\"]\n", condense_display ? t->id : n->nodeid, rmsummary_resource_to_str("gpus", n->resources_requested->cores, 0));
+				printf("\tresMem%d [style=filled, color=white, label=\"memory: %s\"]\n", condense_display ? t->id : n->nodeid, rmsummary_resource_to_str("memory", n->resources_requested->memory, 1));
+				printf("\tworkDirFtprnt%d [style=filled, color=white, label=\"footprint: %s\"]\n", condense_display ? t->id : n->nodeid, rmsummary_resource_to_str("disk", n->resources_requested->disk, 1));
 				printf("\tcores%d -> resMem%d -> workDirFtprnt%d [color=white]", condense_display ? t->id : n->nodeid, condense_display ? t->id : n->nodeid, condense_display ? t->id : n->nodeid);
 
 				//Source Files
 				list_first_item(n->source_files);
-				i = 0;
 				while((f = list_next_item(n->source_files))) {
 					fn = f->filename;
 					e = hash_table_lookup(g, fn);
 					if(e) {
-						i++;
 						printf("\tsrc_%d_%d [label=\"%s\", style=unfilled, color=purple, shape=box];\n", condense_display ? t->id : n->nodeid, e->id, e->name);
 						printf("\tsrc_%d_%d -> N%d;\n", condense_display ? t->id : n->nodeid, e->id, condense_display ? t->id : n->nodeid);
 					}
@@ -810,12 +807,10 @@ void dag_to_dot(struct dag *d, int condense_display, int change_size, int with_l
 
 				//Target Files
 				list_first_item(n->target_files);
-				j = 0;
 				while((f = list_next_item(n->target_files))) {
 					fn = f->filename;
 					e = hash_table_lookup(g, fn);
 					if(e) {
-						j++;
 						printf("\ttar_%d_%d [label=\"%s\", style=dotted, color=purple, shape=box];\n", condense_display ? t->id : n->nodeid, e->id, e->name);
 						printf("\tN%d -> tar_%d_%d;\n", condense_display ? t->id : n->nodeid, condense_display ? t->id : n->nodeid, e->id);
 
@@ -1183,6 +1178,12 @@ struct jx *category_allocation_to_json(category_allocation_t c) {
 		case CATEGORY_ALLOCATION_ERROR:
 			return jx_string("error");
 			break;
+		case CATEGORY_ALLOCATION_GREEDY_BUCKETING:
+			return jx_string("greedy bucketing");
+			break;
+		case CATEGORY_ALLOCATION_EXHAUSTIVE_BUCKETING:
+			return jx_string("exhaustive bucketing");
+			break;
 	}
 	return jx_null();
 }
