@@ -37,7 +37,7 @@ struct dag_file *makeflow_module_lookup_fail_dir(struct dag *d, struct batch_que
 		}
 		return f;
 	} else {
-		if (!batch_fs_stat(q, path, &buf)) {
+		if (!stat(path, &buf)) {
 			debug(D_MAKEFLOW_HOOK,
 					"skipping %s since it already exists",
 					path);
@@ -61,7 +61,7 @@ int makeflow_module_move_fail_file(struct dag *d, struct dag_node *n, struct bat
 			return MAKEFLOW_HOOK_SUCCESS;
 		}
 
-		if (batch_fs_rename(q, f->filename, o->filename) < 0) {
+		if (rename(f->filename, o->filename) < 0) {
 			debug(D_MAKEFLOW_HOOK, "Failed to rename %s -> %s: %s",
 					f->filename, o->filename, strerror(errno));
 		} else {
@@ -77,6 +77,36 @@ int makeflow_module_move_fail_file(struct dag *d, struct dag_node *n, struct bat
 	return MAKEFLOW_HOOK_FAILURE;
 }
 
+/* Attempt to link input files into fail dir. Do not fail on failed link. */
+int makeflow_module_link_fail_file(struct dag *d, struct dag_node *n, struct batch_queue *q, struct dag_file *f) {
+	assert(d);
+	assert(n);
+	assert(q);
+	assert(f);
+
+	char *failout = string_format( FAIL_DIR "/%s", n->nodeid, f->filename);
+	struct dag_file *o = makeflow_module_lookup_fail_dir(d, q, failout);
+	if (o) {
+		if(f->state == DAG_FILE_STATE_DELETE){
+			debug(D_MAKEFLOW_HOOK, "File %s has already been deleted by another hook",f->filename);
+			return MAKEFLOW_HOOK_SUCCESS;
+		}
+
+		if (symlink(f->filename, o->filename) < 0) {
+			debug(D_MAKEFLOW_HOOK, "Failed to link %s -> %s: %s",
+					f->filename, o->filename, strerror(errno));
+		} else {
+			debug(D_MAKEFLOW_HOOK, "Linked %s -> %s",
+					f->filename, o->filename);
+			return MAKEFLOW_HOOK_SUCCESS;
+		}
+	} else {
+		fprintf(stderr, "Skipping link %s -> %s", f->filename, failout);
+	}
+	free(failout);
+	return MAKEFLOW_HOOK_SUCCESS;
+}
+
 int makeflow_module_prep_fail_dir(struct dag *d, struct dag_node *n, struct batch_queue *q) {
 	assert(d);
 	assert(n);
@@ -105,7 +135,7 @@ FAILURE:
 	return rc;
 }
 
-static int node_success( void * instance_struct, struct dag_node *n, struct batch_task *task){
+static int node_success( void * instance_struct, struct dag_node *n, struct batch_job *task){
 	struct dag *d = n->d;
 	struct batch_queue *q = makeflow_get_remote_queue();
 
@@ -125,7 +155,7 @@ static int node_success( void * instance_struct, struct dag_node *n, struct batc
 	return rc;
 }
 
-static int node_fail( void * instance_struct, struct dag_node *n, struct batch_task *task){
+static int node_fail( void * instance_struct, struct dag_node *n, struct batch_job *task){
 	struct batch_file *bf = NULL;
 	struct dag_file *df = NULL;
 	int prep_failed =  makeflow_module_prep_fail_dir(n->d, n, task->queue); 
@@ -138,11 +168,12 @@ static int node_fail( void * instance_struct, struct dag_node *n, struct batch_t
 	/* Dump node info */
 	char *info_path = string_format(FAIL_DIR "/INFO.json", n->nodeid);
 	FILE *info_stream = fopen(info_path, "w");
-	free(info_path);
 	if (!info_stream) {
 		debug(D_MAKEFLOW_HOOK, "Failed to create %s: %s", info_path, strerror(errno));
+		free(info_path);
 		return MAKEFLOW_HOOK_FAILURE;
 	}
+	free(info_path);
 	struct jx *info = dag_node_to_jx(n->d, n, 0);
 	if (n->task->info->exited_normally) {
 		jx_insert(info, jx_string("exit_code"), jx_integer(n->task->info->exit_code));
@@ -159,6 +190,8 @@ static int node_fail( void * instance_struct, struct dag_node *n, struct batch_t
 		df = dag_file_lookup_or_create(n->d, bf->outer_name);
 		if(df->type == DAG_FILE_TYPE_TEMP) {
 			makeflow_module_move_fail_file(n->d, n, makeflow_get_queue(n), df);
+		} else {
+			makeflow_module_link_fail_file(n->d, n, makeflow_get_queue(n), df);
 		}
 	}
 
