@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2003-2004 Douglas Thain and the University of Wisconsin
-Copyright (C) 2005- The University of Notre Dame
+Copyright (C) 2022 The University of Notre Dame
 This software is distributed under the GNU General Public License.
 See the file COPYING for details.
 */
@@ -67,7 +67,12 @@ void pfs_process_kill_everyone( int sig )
 	debug(D_NOTICE,"sending myself %d (%s), goodbye!",sig,string_signal(sig));
 	while(1) {
 		signal(sig,SIG_DFL);
-		sigsetmask(~sigmask(sig));
+
+		sigset_t s;
+		sigfillset(&s);
+		sigdelset(&s, sig);
+		sigprocmask(SIG_SETMASK, &s, NULL);\
+
 		kill(getpid(),sig);
 		kill(getpid(),SIGKILL);
 	}
@@ -623,4 +628,4 @@ int pfs_process_setgroups(struct pfs_process *p, size_t size, const gid_t *list)
 	return 0;
 }
 
-/* vim: set noexpandtab tabstop=4: */
+/* vim: set noexpandtab tabstop=8: */
