@@ -1,6 +1,6 @@
 /*
 Copyright (C) 2003-2004 Douglas Thain and the University of Wisconsin
-Copyright (C) 2005- The University of Notre Dame
+Copyright (C) 2022 The University of Notre Dame
 This software is distributed under the GNU General Public License.
 See the file COPYING for details.
 */
@@ -558,7 +558,7 @@ int pfs_table::resolve_name(int is_special_syscall, const char *cname, struct pf
 					pname->port = pname->service->get_default_port();
 				}
 				char *hostport=string_format("%s:%d", pname->host, pname->port);
-				strncpy(pname->hostport, hostport, sizeof(pname->hostport));
+				snprintf(pname->hostport, sizeof(pname->hostport), "%s", hostport);
 				free(hostport);
 			}
 
@@ -1095,7 +1095,6 @@ int pfs_table::fstat( int fd, struct pfs_stat *b )
 	return result;
 }
 
-
 int pfs_table::fstatfs( int fd, struct pfs_statfs *buf )
 {
 	CHECK_FD(fd);
@@ -1549,6 +1548,33 @@ int pfs_table::stat( const char *n, struct pfs_stat *b )
 	return result;
 }
 
+int pfs_table::statx( const char *name, int flags, unsigned int mask, struct pfs_statx *b )
+{
+	pfs_name pname;
+	int result = -1;
+
+	int follow_links = !(flags & AT_SYMLINK_NOFOLLOW);
+
+	/* You don't need to have read permission on a file to stat it. */
+	if(resolve_name(0,name,&pname,F_OK,follow_links)) {
+		result = pname.service->statx(&pname,flags,mask,b);
+		if(result>=0 && b->stx_blksize < 1) {
+			b->stx_blksize = pname.service->get_block_size();
+		} else if(errno==ENOENT && !pname.hostport[0]) {
+			struct pfs_stat stat_b;
+			struct pfs_statx statx_b;
+			pfs_service_emulate_stat(&pname,&stat_b);
+			COPY_STAT_TO_STATX(stat_b, statx_b);
+			memcpy(b, &statx_b, sizeof(*b));
+			b->stx_mode = S_IFDIR | 0555;
+			result = 0;
+		}
+	}
+
+	return result;
+}
+
+
 int pfs_table::statfs( const char *n, struct pfs_statfs *b )
 {
 	pfs_name pname;
@@ -2652,4 +2678,4 @@ int pfs_table::mmap_delete( uintptr_t logical_addr, size_t length )
 	return 0;
 }
 
-/* vim: set noexpandtab tabstop=4: */
+/* vim: set noexpandtab tabstop=8: */
