@@ -65,7 +65,8 @@ class Manager(object):
     # @param port       The port number to listen on. If zero, then a random port is chosen. A range of possible ports (low, hight) can be also specified instead of a single integer. Default is 9123
     # @param name       The project name to use.
     # @param shutdown   Automatically shutdown workers when manager is finished. Disabled by default.
-    # @param run_info_path Directory to write log (and staging if staging_path not given) files per run. If None, defaults to "vine-run-info"
+    # @param run_info_path      Directory to archive workflow log directories, it is the upper level directory to run_info_template. If None, defaults to "vine-run-info"
+    # @param run_info_template  See run_info_path. If None, defaults by a %Y-%m-%dT%H%M%S format.
     # @param staging_path Directory to write temporary files. Defaults to run_info_path if not given.
     # @param ssl        A tuple of filenames (ssl_key, ssl_cert) in pem format, or True.
     #                   If not given, then TSL is not activated. If True, a self-signed temporary key and cert are generated.
@@ -78,6 +79,7 @@ class Manager(object):
                  name=None,
                  shutdown=False,
                  run_info_path="vine-run-info",
+                 run_info_template=None,
                  staging_path=None,
                  ssl=None,
                  init_fn=None,
@@ -111,8 +113,13 @@ class Manager(object):
             self._info_widget = JupyterDisplay(interval=status_display_interval)
 
         try:
+            # Set an internal variable in the C code.
+            # No need to unset it explicitly as it doesn't rely on environment variables.
             if run_info_path:
-                self.set_runtime_info_path(run_info_path)
+                cvine.vine_set_runtime_info_path(run_info_path)
+
+            if run_info_template:
+                cvine.vine_set_runtime_info_template(run_info_template)
 
             self._stats = cvine.vine_stats()
             self._stats_hierarchy = cvine.vine_stats()
@@ -257,7 +264,7 @@ class Manager(object):
     # Get the caching directory of the manager
     @property
     def cache_directory(self):
-        return cvine.vine_get_path_staging(self._taskvine, None)
+        return cvine.vine_get_path_cache(self._taskvine, None)
 
     ##
     # Get manager statistics.
@@ -554,14 +561,6 @@ class Manager(object):
     def set_property(self, name, value):
         cvine.vine_set_property(self._taskvine, name, value)
 
-    ##
-    # Specify a directory to write logs and staging files.
-    #
-    # @param self     Reference to the current manager object.
-    # @param dirname  A directory name
-    def set_runtime_info_path(self, dirname):
-        cvine.vine_set_runtime_info_path(dirname)
-
     ##
     # Add a mandatory password that each worker must present.
     #
@@ -926,53 +925,93 @@ class Manager(object):
     # @param self            Reference to the current manager object.
     # @param library_name    Name of the Library to be created
     # @param function_list   List of all functions to be included in the library
-    # @param poncho_env      Name of an already prepared poncho environment
+    # @param poncho_env      Name of an already prepared poncho environment or a conda environment
     # @param init_command    A string describing a shell command to execute before the library task is run
     # @param add_env         Whether to automatically create and/or add environment to the library
-    # @returns               A task to be used with @ref ndcctools.taskvine.manager.Manager.install_library.
     # @param hoisting_modules  A list of modules imported at the preamble of library, including packages, functions and classes.
-    def create_library_from_functions(self, library_name, *function_list, poncho_env=None, init_command=None, add_env=True, hoisting_modules=None):
+    # @param exec_mode       Execution mode that the library should use to run function calls. Either 'direct' or 'fork'
+    # @param library_context_info   A list containing [library_context_func, library_context_args, library_context_kwargs]. Used to create the library context on remote nodes.
+    # @returns               A task to be used with @ref ndcctools.taskvine.manager.Manager.install_library.
+    def create_library_from_functions(self, library_name, *function_list, poncho_env=None, init_command=None, add_env=True, hoisting_modules=None, exec_mode='fork', library_context_info=None):
         # Delay loading of poncho until here, to avoid bringing in poncho dependencies unless needed.
         # Ensure poncho python library is available.
         from ndcctools.poncho import package_serverize
 
-        # Positional arguments are the list of functions to include in the library.
-        # Create a unique hash of a combination of function names and bodies.
-        functions_hash = package_serverize.generate_functions_hash(function_list, hoisting_modules)
+        # Check if the library is empty
+        if len(function_list) == 0:
+            raise ValueError('A library cannot have 0 functions.')
+
+        # Check if exec_mode is valid
+        # Currently taskvine only supports 'fork' and 'direct'.
+        if exec_mode != 'fork' and exec_mode != 'direct':
+            raise ValueError(f'A library cannot have exec_mode as {exec_mode}. Only "fork" and "direct" are supported.')
+
+        # Create a unique hash of a library from all information that determine a library's uniqueness.
+        library_hash = package_serverize.generate_library_hash(library_name=library_name,
+                                                               function_list=function_list,
+                                                               poncho_env=poncho_env,
+                                                               init_command=init_command,
+                                                               add_env=add_env,
+                                                               exec_mode=exec_mode,
+                                                               hoisting_modules=hoisting_modules,
+                                                               library_context_info=library_context_info)
 
         # Create path for caching library code and environment based on function hash.
-        library_cache_path = f"{self.cache_directory}/vine-library-cache/{functions_hash}"
-        library_code_path = f"{library_cache_path}/library_code.py"
-
-        # Don't create a custom poncho environment if it's already given.
-        if poncho_env:
-            library_env_path = poncho_env
-        else:
-            library_env_path = f"{library_cache_path}/library_env.tar.gz"
+        library_cache_dir_name = "vine-library-cache"
+        library_code_name = "library_code.py"
+        library_info_name = "library_info.clpk"
+        library_cache_path = os.path.join(self.cache_directory, library_cache_dir_name, library_hash)
+        library_code_path = os.path.join(library_cache_path, library_code_name)
+        library_info_path = os.path.join(library_cache_path, library_info_name)
 
         # If library cache directory doesn't exist, create it.
         pathlib.Path(library_cache_path).mkdir(mode=0o755, parents=True, exist_ok=True)
 
+        # Don't create a custom poncho environment if it's already given.
+        if poncho_env:
+            library_env_path = poncho_env
+            if not os.path.isfile(library_env_path) and add_env is True:
+                # library_env_path must be the name of a conda environment
+                conda_env_name = library_env_path
+                library_env_path = os.path.join(library_cache_path, library_env_path)
+                library_env_path += '.tar.gz'
+                if not os.path.isfile(library_env_path):
+                    from ndcctools.poncho.package_create import pack_env
+                    pack_env(conda_env_name, library_env_path)
+        else:
+            default_poncho_tarball = 'library_env.tar.gz'
+            library_env_path = os.path.join(library_cache_path, default_poncho_tarball)
+
         # If the library code and environment exist, move on to creating the Library Task.
         # Else create them in the relevant paths.
         if not (os.path.isfile(library_code_path) and os.path.isfile(library_env_path)):
-            # Don't create a new poncho environment tarball is one is already provided or
-            # user explicitly tells not to via `add_env`.
+            # Don't create a new poncho environment tarball if one is already provided or
+            # user explicitly tells not to via `add_env` or the poncho tarball already exists.
             need_pack = True
-            if poncho_env or not add_env:
+            if poncho_env or not add_env or os.path.isfile(library_env_path):
                 need_pack = False
 
-            # create library code and environment, if appropriate
-            package_serverize.serverize_library_from_code(library_cache_path, function_list, library_name, need_pack=need_pack, hoisting_modules=hoisting_modules)
+            # create library code
+            # environment is also created if need_pack is True
+            package_serverize.generate_library(library_cache_path=library_cache_path,
+                                               library_code_path=library_code_path,
+                                               library_env_path=library_env_path,
+                                               library_info_path=library_info_path,
+                                               functions=function_list,
+                                               library_name=library_name,
+                                               need_pack=need_pack,
+                                               exec_mode=exec_mode,
+                                               hoisting_modules=hoisting_modules,
+                                               library_context_info=library_context_info)
 
             # enable correct permissions for library code
             os.chmod(library_code_path, 0o775)
 
         # Create Task to execute the Library and prepend it with some setup code if needed.
         if init_command:
-            t = LibraryTask(f"{init_command} python ./library_code.py", library_name)
+            t = LibraryTask(f"{init_command} python {library_code_name}", library_name)
         else:
-            t = LibraryTask("python ./library_code.py", library_name)
+            t = LibraryTask(f"python {library_code_name}", library_name)
 
         # Declare the environment if needed.
         if add_env:
@@ -980,8 +1019,13 @@ class Manager(object):
             t.add_environment(f)
 
         # Declare the library code as an input.
-        f = self.declare_file(library_code_path, cache=True)
-        t.add_input(f, "library_code.py")
+        f = self.declare_file(library_code_path, cache=True, peer_transfer=True)
+        t.add_input(f, library_code_name)
+        f = self.declare_file(library_info_path, cache=True, peer_transfer=True)
+        t.add_input(f, library_info_name)
+
+        # Register execution mode of functions in this library
+        t.set_function_exec_mode_from_string(exec_mode)
         return t
 
     ##
@@ -1721,6 +1765,14 @@ class Manager(object):
     def log_debug_app(self, entry):
         cvine.vine_log_debug_app(self._taskvine, entry)
 
+    ##
+    # Gets the number of replicas of a file.
+    #
+    # @param self   The manager to register this file
+    # @param file   The File object
+    def get_file_replica_count(self, file):
+        return cvine.vine_file_replica_count(self._taskvine, file._file)
+
 
 ##
 # @class ndcctools.taskvine.manager.Factory
@@ -1806,6 +1858,12 @@ class Factory(object):
         "condor-requirements",
     ]
 
+    # subset of command line options that need special handling once the factory object has been created.
+    _config_init_options = [
+        "batch-type",
+        "manager-name",
+    ]
+
     ##
     # Create a factory for the given batch_type and manager name.
     #
@@ -1835,20 +1893,19 @@ class Factory(object):
             pathlib.Path.mkdir(pathlib.Path(self._opts["scratch-dir"]), exist_ok=True, parents=True)
 
     def _set_manager(self, batch_type, manager, manager_host_port, manager_name):
-        if not (manager or manager_host_port or manager_name):
-            raise ValueError("Either manager, manager_host_port, or manager_name or manager should be specified.")
-
-        if manager_name:
-            self._opts["manager-name"] = manager_name
-
         if manager:
+            if manager.using_ssl:
+                self._opts["ssl"] = True
             if batch_type == "local":
                 manager_host_port = f"localhost:{manager.port}"
             elif manager.name:
-                self._opts["manager-name"] = manager_name
-
-            if manager.using_ssl:
-                self._opts["ssl"] = True
+                if manager_name:
+                    if manager.name != manager_name:
+                        RuntimeError(
+                            "The manager and Factory were assigned different names ({manager.name}, {manager_name})"
+                        )
+                else:
+                    manager_name = manager.name
 
         if manager_host_port:
             try:
@@ -1858,6 +1915,10 @@ class Factory(object):
                 return
             except (TypeError, ValueError):
                 raise ValueError("manager_host_port is not of the form HOST:PORT")
+        elif manager_name:
+            self._opts["manager-name"] = manager_name
+        else:
+            raise ValueError("Either manager, manager_host_port, or manager_name or manager should be specified.")
 
     def _find_exe(self, path, default):
         if path is None:
@@ -1908,6 +1969,9 @@ class Factory(object):
             else:
                 raise AttributeError("{} is not a supported option".format(name))
         else:
+            if name_with_hyphens in Factory._config_init_options:
+                raise AttributeError("{} cannot be changed after the factory initial configuration.".format(name))
+
             if name_with_hyphens in Factory._command_line_options:
                 self._opts[name_with_hyphens] = value
             else:
