@@ -279,6 +279,15 @@ class Task(object):
     def set_function_slots(self, nslots):
         return cvine.vine_task_set_function_slots(self._task, nslots)
 
+    ##
+    # Set the execution mode of functions in a library.
+    # This is not needed for regular tasks.
+    #
+    # @param self       Reference to the current task object.
+    # @param exec_mode  The execution mode of functions in a library. Either 'fork' or 'direct'.
+    def set_function_exec_mode_from_string(self, exec_mode):
+        return cvine.vine_task_set_function_exec_mode_from_string(self._task, exec_mode)
+
     ##
     # Set the worker selection scheduler for task.
     #
@@ -574,6 +583,15 @@ class Task(object):
     def command(self):
         return cvine.vine_task_get_command(self._task)
 
+    ##
+    # Get the state of the task.
+    # @code
+    # >>> print(t.command)
+    # @endcode
+    @property
+    def state(self):
+        return cvine.vine_task_get_state(self._task)
+
     ##
     # Get the standard output of the task. Must be called only after the task
     # completes execution.
@@ -1182,4 +1200,5 @@ class LibraryTask(Task):
         self._manager_will_free = True
         self.provides_library(library_name)
 
+
 # vim: set sts=4 sw=4 ts=4 expandtab ft=python:
