@@ -312,12 +312,21 @@ If unset, the library will runs as many functions as it has cores available.
 */
 void vine_task_set_function_slots(struct vine_task *t, int nslots);
 
+/** Set the execution mode of functions inside a library.
+A mode can either be "fork" where the library forks and executes functions, or
+"direct" where the library executes a function in its memory space.
+Note that "direct" will limit the number of function slots in a library to 1.
+@param t                A library object.
+@param exec_mode        A string denoting the execution mode of the library.
+*/
+void vine_task_set_function_exec_mode_from_string(struct vine_task *t, const char *exec_mode);
+
 /** Add a general file object as a input to a task.
 @param t A task object.
 @param f A file object, created by @ref vine_declare_file, @ref vine_declare_url, @ref vine_declare_buffer, @ref
 vine_declare_mini_task.
 @param remote_name The name of the file as it should appear in the task's sandbox.
-@param flags May be zero or more @ref vine_mount_flags_t or'd together. See @ref vine_task_add_input.
+@param flags May be zero or more @ref vine_mount_flags_t or'd together. See @ref vine_task_add_output.
 @return True on success, false on failure.
 */
 
@@ -442,7 +451,6 @@ regardless of the priority.
 @param t A task object.
 @param priority The priority of the task.
 */
-
 void vine_task_set_priority(struct vine_task *t, double priority);
 
 /** Specify an environment variable to be added to the task.
@@ -468,6 +476,13 @@ also written to that directory.
 
 int vine_task_set_monitor_output(struct vine_task *t, const char *monitor_output);
 
+/** Get the state line of the task.
+@param t A task object.
+@return a string of the task's state.
+*/
+
+const char *vine_task_get_state(struct vine_task *t);
+
 /** Get the command line of the task.
 @param t A task object.
 @return The command line set by @ref vine_task_create.
@@ -714,6 +729,13 @@ const char *vine_file_source(struct vine_file *f);
 */
 vine_file_type_t vine_file_type(struct vine_file *f);
 
+/** Get the number of replicas of a file.
+@param m A manager object
+@param f A file object.
+@return The number of replicas of the file.
+*/
+int vine_file_replica_count(struct vine_manager *m, struct vine_file *f);
+
 /** Declare a file object from a local file
 @param m A manager object
 @param source The path of the file on the local filesystem
@@ -866,6 +888,16 @@ be delete at the manager's site after it is not needed by the workflow (@ref vin
 struct vine_file *vine_declare_starch(
 		struct vine_manager *m, struct vine_file *f, vine_cache_level_t cache, vine_file_flags_t flags);
 
+/** Set the Unix mode permission bits of a declared file.
+Sets (or overrides) the Unix mode permissions of any file object,
+as the application will see it.  This applies to any file type,
+but is particularly useful for controlling buffers, urls, and mini-tasks
+that do not inherently contain mode bits.
+@param f A file object of any kind.
+@param mode The Unix mode bits to be applied to the file.
+*/
+void vine_file_set_mode( struct vine_file *f, int mode );
+
 /** Fetch the contents of a file.
 The contents of the given file will be loaded from disk or pulled back from the cluster
 and loaded into manager memory.  This is particularly useful for temporary files and mini-tasks
@@ -1449,6 +1481,11 @@ void vine_initialize_categories(struct vine_manager *m, struct rmsummary *max, c
 */
 void vine_set_runtime_info_path(const char *path);
 
+/** Sets the directory where a workflow-specific runtime logs are directly written into.
+@param dir A directory
+*/
+void vine_set_runtime_info_template(const char *template);
+
 /** Adds a custom APPLICATION entry to the debug log.
 @param m     Reference to the current manager object.
 @param entry A custom debug message.
