@@ -14,6 +14,7 @@ See the file COPYING for details.
 #define VINE_FILE_REPLICA_TABLE_H
 
 #include "taskvine.h"
+#include "set.h"
 #include "vine_file_replica.h"
 #include "vine_worker_info.h"
 
@@ -27,7 +28,7 @@ struct vine_file_replica *vine_file_replica_table_lookup(struct vine_worker_info
 
 struct vine_worker_info *vine_file_replica_table_find_worker(struct vine_manager *q, const char *cachename);
 
-int vine_file_replica_table_replicate(struct vine_manager *q, struct vine_file *f);
+int vine_file_replica_table_replicate(struct vine_manager *q, struct vine_file *f, struct set *sources, int to_find);
 
 int vine_file_replica_table_exists_somewhere( struct vine_manager *q, const char *cachename );
 
