@@ -26,6 +26,8 @@ See the file COPYING for details.
 #include <string.h>
 #include <unistd.h>
 
+void vine_task_set_function_exec_mode(struct vine_task *t, vine_task_func_exec_mode_t exec_mode);
+
 struct vine_task *vine_task_create(const char *command_line)
 {
 	struct vine_task *t = malloc(sizeof(*t));
@@ -210,6 +212,8 @@ struct vine_task *vine_task_copy(const struct vine_task *task)
 		vine_task_set_library_required(new, task->needs_library);
 	if (task->provides_library)
 		vine_task_set_library_provided(new, task->provides_library);
+	if (task->func_exec_mode)
+		vine_task_set_function_exec_mode(new, task->func_exec_mode);
 	if (task->tag)
 		vine_task_set_tag(new, task->tag);
 	if (task->category)
@@ -313,6 +317,20 @@ void vine_task_set_function_slots(struct vine_task *t, int nslots)
 	t->function_slots_requested = nslots;
 }
 
+void vine_task_set_function_exec_mode(struct vine_task *t, vine_task_func_exec_mode_t exec_mode)
+{
+	if (t->provides_library) {
+		t->func_exec_mode = exec_mode;
+	}
+}
+
+void vine_task_set_function_exec_mode_from_string(struct vine_task *t, const char *exec_mode)
+{
+	if (exec_mode && t->provides_library) {
+		t->func_exec_mode = vine_task_func_exec_mode_from_string(exec_mode);
+	}
+}
+
 void vine_task_set_env_var(struct vine_task *t, const char *name, const char *value)
 {
 	if (value) {
@@ -550,6 +568,8 @@ int vine_task_add_output(struct vine_task *t, struct vine_file *f, const char *r
 		return 0;
 	}
 
+	f->state = VINE_FILE_STATE_PENDING;
+
 	struct vine_mount *m = vine_mount_create(f, remote_name, flags, 0);
 
 	list_push_tail(t->output_mounts, m);
@@ -794,6 +814,11 @@ const char *vine_task_get_hostname(struct vine_task *t)
 	return t->hostname;
 }
 
+const char *vine_task_get_state(struct vine_task *t)
+{
+	return vine_task_state_to_string(t->state);
+}
+
 #define METRIC(x) \
 	if (!strcmp(name, #x)) \
 		return t->x;
@@ -977,3 +1002,16 @@ char *vine_task_to_json(struct vine_task *t)
 	buffer_free(&b);
 	return json;
 }
+
+/* Converts a string to a valid vine_task_func_exec_mode_t.
+ * Returns VINE_TASK_FUNC_EXEC_MODE_INVALID if there's no valid mode for the string. */
+vine_task_func_exec_mode_t vine_task_func_exec_mode_from_string(const char *exec_mode)
+{
+	if (!strncmp(exec_mode, "direct", strlen("direct"))) {
+		return VINE_TASK_FUNC_EXEC_MODE_DIRECT;
+	}
+	if (!strncmp(exec_mode, "fork", strlen("fork"))) {
+		return VINE_TASK_FUNC_EXEC_MODE_FORK;
+	}
+	return VINE_TASK_FUNC_EXEC_MODE_INVALID;
+}
