@@ -228,6 +228,12 @@ we break once a message would overflow the window.
 
 void deliver_async_messages(struct link *l)
 {
+	/* If no pending messages, return right away. */
+	int messages = list_size(pending_async_messages);
+	if (messages < 1)
+		return;
+
+	/* Determine how much space is available for sending */
 	int recv_window;
 	int send_window;
 	link_window_get(l, &send_window, &recv_window);
@@ -235,14 +241,19 @@ void deliver_async_messages(struct link *l)
 	int bytes_in_buffer = link_get_buffer_bytes(l);
 	int bytes_available = send_window - bytes_in_buffer;
 
-	int messages = list_size(pending_async_messages);
 	int visited;
-	char *message;
 
+	/* Consider each message in the pending queue: */
 	for (visited = 0; visited < messages; visited++) {
-		message = list_peek_head(pending_async_messages);
+		char *message = list_peek_head(pending_async_messages);
 		int message_size = strlen(message);
-		if (message_size < bytes_available) {
+		/*
+		If the message fits in the available space, send it.
+		OR: If it is larger than the whole window, send it anyway
+		because we will have to block one way or the other.
+		Otherwise, stop here and return later.
+		*/
+		if (message_size < bytes_available || message_size > send_window) {
 			message = list_pop_head(pending_async_messages);
 			bytes_available -= message_size;
 			debug(D_VINE, "tx: %s", message);
@@ -863,6 +874,14 @@ static struct vine_task *do_task_body(struct link *manager, int task_id, time_t
 			task->function_slots_requested = n;
 			/* Also set the total number determined by the manager. */
 			task->function_slots_total = n;
+		} else if (sscanf(line, "func_exec_mode %" PRId64, &n) == 1) {
+			vine_task_func_exec_mode_t func_exec_mode = n;
+			if (func_exec_mode == VINE_TASK_FUNC_EXEC_MODE_INVALID) {
+				debug(D_VINE | D_NOTICE, "invalid func_exec_mode from manager: %s", line);
+				vine_task_delete(task);
+				return 0;
+			}
+			task->func_exec_mode = func_exec_mode;
 		} else if (sscanf(line, "infile %s %s %d", localname, taskname_encoded, &flags)) {
 			url_decode(taskname_encoded, taskname, VINE_LINE_MAX);
 			vine_hack_do_not_compute_cached_name = 1;
@@ -1544,10 +1563,25 @@ static int check_library_startup(struct vine_process *p)
 	struct jx *response = jx_parse_string(buffer);
 
 	const char *name = jx_lookup_string(response, "name");
+	int taskid = jx_lookup_integer(response, "taskid");
+	const char *exec_mode = jx_lookup_string(response, "exec_mode");
 
-	int ok = 0;
-	if (!strcmp(name, p->task->provides_library)) {
-		ok = 1;
+	int ok = 1;
+
+	if (!name || !taskid || !exec_mode) {
+		ok = 0;
+	} else {
+		vine_task_func_exec_mode_t converted_exec_mode = vine_task_func_exec_mode_from_string(exec_mode);
+
+		if (!p->task->provides_library || strcmp(name, p->task->provides_library)) {
+			ok = 0;
+		}
+		if (taskid != p->task->task_id) {
+			ok = 0;
+		}
+		if (p->task->func_exec_mode && converted_exec_mode != p->task->func_exec_mode) {
+			ok = 0;
+		}
 	}
 	if (response) {
 		jx_delete(response);
@@ -1583,8 +1617,7 @@ static void check_libraries_ready(struct link *manager)
 				debug(D_VINE, "Library %s reports ready to execute functions.", library_process->task->provides_library);
 				library_process->library_ready = 1;
 			} else {
-				/* Kill library if the name reported back doesn't match its name or
-				 * if there's any problem. */
+				/* Kill library if it fails the startup check. */
 				debug(D_VINE,
 						"Library %s task id %" PRIu64 " verification failed (unexpected response). Killing it.",
 						library_process->task->provides_library,
@@ -1714,7 +1747,7 @@ static void vine_worker_serve_manager(struct link *manager)
 				} else if (process_can_run_eventually(p, cache_manager, manager)) {
 					list_push_tail(procs_waiting, p);
 				} else {
-					debug(D_VINE, "No suitable library found for task %d", p->task->task_id);
+					debug(D_VINE, "task does not have necessary resources to run %d", p->task->task_id);
 					forsake_waiting_process(manager, p);
 					task_event++;
 				}
@@ -1828,7 +1861,7 @@ static int vine_worker_serve_manager_by_hostport(const char *host, int port, con
 	vine_workspace_prepare(workspace);
 
 	/* Start the cache manager and scan for existing files. */
-	cache_manager = vine_cache_create(workspace->cache_dir);
+	cache_manager = vine_cache_create(workspace->cache_dir, options->max_transfer_procs);
 	vine_cache_load(cache_manager);
 
 	/* Start the transfer server, which serves up the cache directory. */
